/*
 *     Implementation by Szymon Ksiazek <szymonh20@poczta.fm>
 *
 *     Copyright (C) 2006-9 by the HeKatE Project
 *
 *     HJEd has been develped within the HeKatE Project, 
 *     see http://hekate.ia.agh.edu.pl
 *
 *     This file is part of HJEd.
 *
 *     HJEd is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     HJEd is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with HJEd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package pl.edu.agh.hekate.ard;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;

/**
 * Implementation class of persistence manager
 * 
 * @author Szymon Książek
 *
 */
public class PersistenceManagerImpl implements PersistenceManager
{
	/**
	 * Model instance
	 */
	private ARDModel model = null;
	
	/**
	 * View instance
	 */
	private ARDView view = null;
	
	/**
	 * Main frame instance
	 */
	private MainFrame mainFrame = null;
	
	/**
	 * true if autosave mode is enabled, false otherwise
	 */
	private boolean isAutosave = false;
	
	/**
	 * specifies if actual state of graphs is saved
	 */
	private boolean isStateSaved = false;
	
	/**
	 * file path to file used to save graphs
	 */
	private String filePath = "";
	
	
	/**
	 * Constructor
	 * 
	 * @param model model instance
	 * @param view	view instance
	 */
	public PersistenceManagerImpl(ARDModel model, ARDView view, MainFrame mainFrame)
	{
		this.model = model;
		this.view = view;
		this.mainFrame = mainFrame;
		
		isAutosave = false;
		isStateSaved = true;
		
		filePath = "";
	}
	
	public void open()
	{
		JFileChooser fileChooser = new JFileChooser();
		
		if (filePath != "")
		{
			fileChooser.setCurrentDirectory(new File(filePath));
		}
		
		fileChooser.setDialogTitle("Open");
		
		fileChooser.setFileFilter(new XMLFilter());
		
		fileChooser.setAcceptAllFileFilterUsed(false);
		
		File currentDirectory = fileChooser.getCurrentDirectory();
		
		boolean correctFile = false;
		
		// until user use correct file
		while (!correctFile)
		{
			correctFile = true;
			
			fileChooser.setCurrentDirectory(currentDirectory);
			
			int returnValue = fileChooser.showDialog(null, "Open");
			
			if (returnValue == JFileChooser.APPROVE_OPTION)
			{
				List<Attribute> newAttributesList = new ArrayList<Attribute>();
				List<Property> newPropertiesList = new ArrayList<Property>();
				List<Transform> newTransformsList = new ArrayList<Transform>();
				List<Dependency> newDependenciesList = new ArrayList<Dependency>();
				
				filePath = fileChooser.getSelectedFile().getAbsolutePath();
				
				try
				{
					DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
					
					DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
					
					EntityResolver entityResolver = new HmlEntityResolver();
					
					// set entity resolver
					docBuilder.setEntityResolver(entityResolver);
					
					Document doc = docBuilder.parse(new File(filePath));
					
					doc.getDocumentElement().normalize();
					
					// load attributes
					NodeList listOfAttributesSets = doc.getElementsByTagName("attribute_set");
					
					for(int i = 0 ; i < listOfAttributesSets.getLength() ; i++)
					{
						Node attributeSetNode = listOfAttributesSets.item(i);
						
						if(attributeSetNode.getNodeType() == Node.ELEMENT_NODE)
						{
							Element attributeSetElement = (Element)attributeSetNode;
							
							NodeList listOfAttributes = attributeSetElement.getElementsByTagName("att");
							
							for(int j = 0 ; j < listOfAttributes.getLength() ; j++)
							{
								Node attributeNode = listOfAttributes.item(j);
								
								if(attributeNode.getNodeType() == Node.ELEMENT_NODE)
								{
									Element attributeElement = (Element)attributeNode;
									
									newAttributesList.add(new Attribute(attributeElement.getAttribute("name")));
								}
							}
						}
					}
					
					// load properties
					NodeList listOfPropertiesSets = doc.getElementsByTagName("property_set");
					
					for(int i = 0 ; i < listOfPropertiesSets.getLength() ; i++)
					{
						Node propertySetNode = listOfPropertiesSets.item(i);
						
						if(propertySetNode.getNodeType() == Node.ELEMENT_NODE)
						{
							Element propertySetElement = (Element)propertySetNode;
							
							NodeList listOfProperties = propertySetElement.getElementsByTagName("property");
							
							for(int j = 0 ; j < listOfProperties.getLength() ; j++)
							{
								Node propertyNode = listOfProperties.item(j);
								
								if(propertyNode.getNodeType() == Node.ELEMENT_NODE)
								{
									Element propertyElement = (Element)propertyNode;
									
									List<Attribute> propertyAttributesList = new ArrayList<Attribute>();
									
									NodeList listOfPropertyAttributes = propertyElement.getElementsByTagName("attref");
									
									for(int k = 0 ; k < listOfPropertyAttributes.getLength() ; k++)
									{
										Node propertyAttributeNode = listOfPropertyAttributes.item(k);
										
										if(propertyAttributeNode.getNodeType() == Node.ELEMENT_NODE)
										{
											Element propertyAttributeElement = (Element)propertyAttributeNode;
											
											propertyAttributesList.add(newAttributesList.get(Integer.parseInt(propertyAttributeElement.getAttribute("ref").substring(propertyAttributeElement.getAttribute("ref").lastIndexOf("_") + 1))));
										}
									}
									
									newPropertiesList.add(new Property(propertyAttributesList));
								}
							}
						}
					}
					
					// load transforms
					NodeList listOfTransformsSets = doc.getElementsByTagName("tph");
					
					for(int i = 0 ; i < listOfTransformsSets.getLength() ; i++)
					{
						Node transformSetNode = listOfTransformsSets.item(i);
						
						if(transformSetNode.getNodeType() == Node.ELEMENT_NODE)
						{
							Element transformSetElement = (Element)transformSetNode;
							
							NodeList listOfTransforms = transformSetElement.getElementsByTagName("trans");
							
							for(int j = 0 ; j < listOfTransforms.getLength() ; j++)
							{
								Node transformNode = listOfTransforms.item(j);
								
								if(transformNode.getNodeType() == Node.ELEMENT_NODE)
								{
									Element transformElement = (Element)transformNode;
					
									Property sourceProperty = newPropertiesList.get(Integer.parseInt(transformElement.getAttribute("src").substring(transformElement.getAttribute("src").lastIndexOf("_") + 1)));
									Property destinyProperty = newPropertiesList.get(Integer.parseInt(transformElement.getAttribute("dst").substring(transformElement.getAttribute("dst").lastIndexOf("_") + 1)));
									
									newTransformsList.add(new Transform(sourceProperty, destinyProperty));								
								}
							}
						}
					}
					
					// load dependencies
					NodeList listOfDependenciesSets = doc.getElementsByTagName("ard");
					
					for(int i = 0 ; i < listOfDependenciesSets.getLength() ; i++)
					{
						Node dependencySetNode = listOfDependenciesSets.item(i);
						
						if(dependencySetNode.getNodeType() == Node.ELEMENT_NODE)
						{
							Element dependencySetElement = (Element)dependencySetNode;
							
							NodeList listOfDependencies = dependencySetElement.getElementsByTagName("dep");
							
							for(int j = 0 ; j < listOfDependencies.getLength() ; j++)
							{
								Node dependencyNode = listOfDependencies.item(j);
								
								if(dependencyNode.getNodeType() == Node.ELEMENT_NODE)
								{
									Element dependencyElement = (Element)dependencyNode;
					
									Property independentProperty = newPropertiesList.get(Integer.parseInt(dependencyElement.getAttribute("independent").substring(dependencyElement.getAttribute("independent").lastIndexOf("_") + 1)));
									Property dependentProperty = newPropertiesList.get(Integer.parseInt(dependencyElement.getAttribute("dependent").substring(dependencyElement.getAttribute("dependent").lastIndexOf("_") + 1)));
									
									newDependenciesList.add(new Dependency(independentProperty, dependentProperty));
								}
							}
						}
					}
					
					mainFrame.setTitle("ARD Editor - " + fileChooser.getSelectedFile().getName());
				}
				catch (NoDTDFileException e)
				{
					JOptionPane.showMessageDialog(fileChooser, "I can't find DTD file", "Warning", JOptionPane.WARNING_MESSAGE);
					
					break;
				}
				catch (Exception e)
				{
					// if this is not valid HML file, show file choose dialog one more time
					correctFile = false;
					e.printStackTrace();
					currentDirectory = fileChooser.getCurrentDirectory();
					
					JOptionPane.showMessageDialog(fileChooser, "This is not valid HML file.", "Warning", JOptionPane.WARNING_MESSAGE);
					
					continue;
				}
				
				
				model.setAttributesList(newAttributesList);
				model.setPropertiesList(newPropertiesList);
				model.setTransformsList(newTransformsList);
				model.setDependenciesList(newDependenciesList);
				
				// create nodes list
				ArrayList<GraphNode> newGraphNodesList = new ArrayList<GraphNode>();
				
				for (Property property : newPropertiesList)
				{
					// present only last phase of graphs (most detailed graph)
					if (model.getTransformDestinies(property).size() == 0)
					{
						newGraphNodesList.add(new GraphNode(property));
					}
				}
				
				view.setARDGraphNodes(newGraphNodesList);
				
				isStateSaved = true;
				
				
				view.setARDScale(1.0);
				view.setTPHScale(1.0);
			}
			else
			{
				break;
			}
		}
	}
	
	
	public void saveAs()
	{
		JFileChooser fileChooser = new JFileChooser();
		
		fileChooser.setDialogTitle("Save as");
		
		if (filePath != "")
		{
			fileChooser.setCurrentDirectory(new File(filePath));
		}
		
		int returnValue = fileChooser.showDialog(null, "Save");
		
		if (returnValue == JFileChooser.APPROVE_OPTION)
		{
			filePath = fileChooser.getSelectedFile().getAbsolutePath();
			
			if (!filePath.endsWith(".xml"))
			{
				filePath = filePath + ".xml";
			}
			
			fileChooser.setVisible(false);
			
			fileChooser.invalidate();
			
			save();
		}		
	}
	
	
	public void save()
	{
		// if graphs have not been saved yet
		if (filePath == "")
		{
			saveAs();
		}
		
		if (filePath != "")
		{
			try 
			{        
		        OutputStream fout= new FileOutputStream(filePath);
		        OutputStream bout= new BufferedOutputStream(fout);
		        OutputStreamWriter out 
		         	= new OutputStreamWriter(bout, "UTF-8");
		        
		        out.write("<?xml version=\"1.0\" ");
		        out.write("encoding=\"utf-8\" standalone=\"no\"?>\r\n"); 
		        out.write("<!DOCTYPE hml SYSTEM \"hml.dtd\">\r\n");
		        out.write("<hml>\r\n");
		        out.write("    <type_set>\n");
		        out.write("    </type_set>\r\n");
		    
		        
		        // save attributes
		        out.write("    <attribute_set>\r\n");
		        
		        List<Attribute> attributesList = model.getAttributesList();
		        
		        for (int i = 0 ; i < attributesList.size() ; i++)
		        {
		        	Attribute attribute = (Attribute)attributesList.get(i);
		        	
		        	out.write("        <att name=\"" + attribute.getName().trim() + "\" id=\"att_" + i + "\" value=\"single\" class=\"ro\"/>\r\n");
		        }
		        out.write("    </attribute_set>\r\n");
		        
		        
		        // save properties
		        out.write("    <property_set>\r\n");
		        
		        List<Property> propertiesList = model.getPropertiesList();
		        
		        for (int i = 0 ; i < propertiesList.size() ; i++)
		        {
		        	Property property = (Property)propertiesList.get(i);
		        	
		        	out.write("        <property id=\"prp_" + i + "\">\r\n");
		        	
		        	List<Attribute> propertyAttributesList = property.getAttributes();
		        	
		        	for (int j = 0 ; j < propertyAttributesList.size() ; j++)
		        	{
		        		Attribute attribute = (Attribute)propertyAttributesList.get(j);
		        		
		        		out.write("            <attref ref=\"att_" + attributesList.indexOf(attribute) + "\"/>\r\n");
		        	}
		        	
		        	
		        	out.write("        </property>\r\n");
		        }
		        
		        out.write("    </property_set>\r\n");
		        
		        
		        // save transforms
		        out.write("    <tph>\r\n");
		        
		        List<Transform> transformsList = model.getTransformsList();
		        
		        for (int i = 0 ; i < transformsList.size() ; i++)
		        {
		        	Transform transform = (Transform)transformsList.get(i);
		        	
		        	Property srcProperty = transform.getSource();
		        	Property dstProperty = transform.getDestiny();
		        	
		        	String transItem = "        <trans src=\"prp_" + propertiesList.indexOf(srcProperty) + "\" dst=\"prp_" + propertiesList.indexOf(dstProperty) + "\"/>\r\n";
		        	
		        	out.write(transItem);
		        }
		        
		        out.write("    </tph>\r\n");
		        
		        
		        // save dependencies
		        out.write("    <ard>\r\n");
		        
		        List<Dependency> dependenciesList = model.getDependenciesList();
		        
		        for (int i = 0 ; i < dependenciesList.size() ; i++)
		        {
		        	Dependency dependency = (Dependency)dependenciesList.get(i);
		        	
		        	Property independentProperty = dependency.getIndependent();
		        	Property dependentProperty = dependency.getDependent();
		        	
		        	String depItem = "        <dep independent=\"prp_" + propertiesList.indexOf(independentProperty) + "\" dependent=\"prp_" + propertiesList.indexOf(dependentProperty) + "\"/>\r\n";
		        	
		        	out.write(depItem);
		        }
		        
		        out.write("    </ard>\r\n");
		        
		        out.write("</hml>\r\n"); 
		        
		        out.flush();  
		        out.close();
		      }
		      catch (UnsupportedEncodingException e) 
		      {
		        e.printStackTrace();
		      }
		      catch (IOException e) 
		      {
		        e.printStackTrace();      
		      }
		
		      isStateSaved = true;
		}
	}


	public boolean isStateSaved() 
	{
		return isStateSaved;
	}
	
	public void setStateSaved(boolean isStateSaved) 
	{
		// if autosave option is enabled, automatically save graphs
		if (!isStateSaved && isAutosave)
		{
			save();
			
			this.isStateSaved = true;
		}
		else if (!isStateSaved && !isAutosave && !mainFrame.getTitle().endsWith("(*)"))
		{
			mainFrame.setTitle(mainFrame.getTitle() + " (*)");
		}
		
		this.isStateSaved = isStateSaved;
		
		if (isStateSaved && mainFrame.getTitle().endsWith("(*)"))
		{
			mainFrame.setTitle(mainFrame.getTitle().substring(0, mainFrame.getTitle().lastIndexOf("(*)")));
		}
	}


	public void setAutosave(boolean isAutosave) 
	{
		// if autosave option is activated, automatically start save operation 
		if (isAutosave)
		{
			save();
			
			// if user chooses "Cancel" option in save dialog, disable autosave mode
			if (filePath != "")
			{
				this.isAutosave = true;
			}
			else
			{
				this.isAutosave = false;
			}
		}
		else
		{
			this.isAutosave = false;
		}
	}


	public boolean isAutosave() 
	{
		return isAutosave;
	}
	
	
	public void setView(ARDView view) 
	{
		this.view = view;
	}
	
	
	/**
	 * Class used while opening graphs from file. It allow to present in open dialog only files with ".xml" extension
	 * 
	 * @author Szymon Książek
	 *
	 */
	private class XMLFilter extends FileFilter
	{
		public String getDescription()
		{
			return "XML files";
		}
		
		public boolean accept(File file)
		{
			if (file.isDirectory()) return true;
			
			String fileName = file.getName().toLowerCase();
			
			if (fileName.endsWith(".xml"))
			{
				return true;
			}
			
			return false;
		}
	}
	
	
	/**
	 * Class that is used as a entity resolver
	 * 
	 * @author Szymon Książek
	 */
	private class HmlEntityResolver implements EntityResolver
	{
		/**
		 * input source that contains content of dtd file
		 */
		private InputSource inputSource = null;
		
		/**
		 * Constructor
		 * 
		 * @throws NoDTDFileException	if DTD file can not be found
		 */
		// use this constructor when run application from directory tree
		/*public HmlEntityResolver() throws NoDTDFileException
		{
			inputSource = null;
			
			try
			{
				InputStream inputStream = null;
				
				inputStream = new FileInputStream(new File("dtd/hml.dtd"));
				
				inputSource = new InputSource(inputStream);
			}
			catch(IOException e)
			{
				e.printStackTrace();
			}
		}*/
		
		// use this constructor when run application from JAR file
		public HmlEntityResolver() throws NoDTDFileException
		{
			inputSource = null;
			
			try
			{
				InputStream inputStream = null;
				
				String jarFilePath = (new File("")).toURI().toURL().toExternalForm() + "ARDEditor.jar";
				URL url = new URL("jar:" + jarFilePath + "!/dtd/hml.dtd");
				inputStream = url.openStream();
				
				inputSource = new InputSource(inputStream);
			}
			catch(MalformedURLException e)
			{
				e.printStackTrace();
			}
			catch(IOException e)
			{
				e.printStackTrace();
			}
		}
		
		public synchronized InputSource resolveEntity(String publicId, String systemId)
		{
		    return inputSource;
		}
	}
	
	
	/**
	 * Exception that is thrown when DTD file can not be found
	 * 
	 * @author Szymon Książek
	 */
	private class NoDTDFileException extends Exception 
	{
		private static final long serialVersionUID = 1L;

		public NoDTDFileException(String reason) 
	    {
	        super(reason);
	    }

	    public NoDTDFileException(Throwable t) 
	    {
	        super(t);
	    }

	    public NoDTDFileException(String reason, Throwable t) 
	    {
	        super(reason, t);
	    }
	}
}
