/*
 *     Implementation by Szymon Ksiazek <szymonh20@poczta.fm>
 *
 *     Copyright (C) 2006-9 by the HeKatE Project
 *
 *     HJEd has been develped within the HeKatE Project, 
 *     see http://hekate.ia.agh.edu.pl
 *
 *     This file is part of HJEd.
 *
 *     HJEd is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     HJEd is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with HJEd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package pl.edu.agh.hekate.ard;

import java.awt.*;

import javax.swing.ImageIcon;

/**
 * Interface of arranger class, which holds informations about visual part of graphs
 * 
 * @author Szymon Książek
 *
 */
public interface Arranger 
{
	/**
	 * Gives color of marked line on ARD graph
	 * 
	 * @return		color of ARD marked line
	 */
	public Color getARDMarkedLineColor();

	
	/**
	 * Sets color of marked line on ARD graph
	 * 
	 * @param markedLineColor	new color of ARD marked line
	 */
	public void setARDMarkedLineColor(Color markedLineColor);

	
	/**
	 * Gives color of unmarked line on ARD graph
	 * 
	 * @return		color of ARD unmarked line
	 */
	public Color getARDUnmarkedLineColor();

	
	/**
	 * Sets color of unmarked line on ARD graph
	 * 
	 * @param markedLineColor	new color of ARD unmarked line
	 */
	public void setARDUnmarkedLineColor(Color unmarkedLineColor);

	
	/**
	 * Gives background color of marked cell on ARD graph
	 * 
	 * @return		background color of ARD marked cell
	 */
	public Color getARDMarkedBackgroundColor();

	
	/**
	 * Sets background color of marked cell on ARD graph
	 * 
	 * @param markedLineColor	new background color of ARD marked cell
	 */
	public void setARDMarkedBackgroundColor(Color markedBackgroundColor);

	
	/**
	 * Gives background color of unmarked cell on ARD graph
	 * 
	 * @return		background color of ARD unmarked cell
	 */
	public Color getARDUnmarkedBackgroundColor();

	
	/**
	 * Sets background color of unmarked cell on ARD graph
	 * 
	 * @param markedLineColor	new background color of ARD unmarked cell
	 */
	public void setARDUnmarkedBackgroundColor(Color unmarkedBackgroundColor);

	
	/**
	 * Gives background color of ARD graph
	 * 
	 * @return		background color of ARD graph
	 */
	public Color getARDBackgroundColor();

	
	/**
	 * Sets background color of ARD graph
	 * 
	 * @param markedLineColor	new background color of ARD graph
	 */
	public void setARDBackgroundColor(Color backgroundColor);

	
	/**
	 * Gives color of marked line on TPH graph
	 * 
	 * @return		color of TPH marked line
	 */
	public Color getTPHMarkedLineColor();

	
	/**
	 * Sets color of marked line on TPH graph
	 * 
	 * @param markedLineColor	new color of TPH marked line
	 */
	public void setTPHMarkedLineColor(Color markedLineColor);

	
	/**
	 * Gives color of unmarked line on TPH graph
	 * 
	 * @return		color of TPH unmarked line
	 */
	public Color getTPHUnmarkedLineColor();

	
	/**
	 * Sets color of unmarked line on TPH graph
	 * 
	 * @param markedLineColor	new color of TPH unmarked line
	 */
	public void setTPHUnmarkedLineColor(Color unmarkedLineColor);

	
	/**
	 * Gives background color of marked cell on TPH graph
	 * 
	 * @return		background color of TPH marked cell
	 */
	public Color getTPHMarkedBackgroundColor();

	
	/**
	 * Sets background color of marked cell on TPH graph
	 * 
	 * @param markedLineColor	new background color of TPH marked cell
	 */
	public void setTPHMarkedBackgroundColor(Color markedBackgroundColor);

	
	/**
	 * Gives background color of unmarked cell on TPH graph
	 * 
	 * @return		background color of TPH unmarked cell
	 */
	public Color getTPHUnmarkedBackgroundColor();

	
	/**
	 * Sets background color of unmarked cell on TPH graph
	 * 
	 * @param markedLineColor	new background color of TPH unmarked cell
	 */
	public void setTPHUnmarkedBackgroundColor(Color unmarkedBackgroundColor);

	
	/**
	 * Gives background color of TPH graph
	 * 
	 * @return		background color of TPH graph
	 */
	public Color getTPHBackgroundColor();
	
	
	/**
	 * Sets background color of TPH graph
	 * 
	 * @param markedLineColor	new background color of TPH graph
	 */
	public void setTPHBackgroundColor(Color backgroundColor);

	
	/**
	 * Specifies if use gradient to create background color of cell in ARD graph
	 * 
	 * @return		true if use gradient, false otherwise
	 */
	public boolean isGradientARDCell();
	
	
	/**
	 * Sets information about requirement of using gradient to create background color of cell in ARD graph
	 * 
	 * @param isGradientARDCell		true to use gradient, false otherwise
	 */
	public void setGradientARDCell(boolean isGradientARDCell);

	
	/**
	 * Specifies if use gradient to create background color of cell in TPH graph
	 * 
	 * @return		true if use gradient, false otherwise
	 */
	public boolean isGradientTPHCell();

	
	/**
	 * Sets information about requirement of using gradient to create background color of cell in TPH graph
	 * 
	 * @param isGradientARDCell		true to use gradient, false otherwise
	 */
	public void setGradientTPHCell(boolean isGradientTPHCell);
	
	
	/**
	 * Specifies if show "F" letter to finalization and "S" letter to split on TPH diagram
	 * 
	 * @return	true if show letters, false otherwise
	 */
	public boolean isShowFinalizationsAndSplits();

	
	/**
	 * Sets information about requirement of using "F" letters for finalizations and "S" letters for splits ok TPH diagram.
	 * 
	 * @param isShowFinalizationsAndSplits		true to show letters, false otherwise
	 */
	public void setShowFinalizationsAndSplits(boolean isShowFinalizationsAndSplits);
	
	
	/**
	 * Specifies if grid should be visible 
	 * 
	 * @return	true if show grid, false otherwise
	 */
	public boolean isShowGrid();

	
	/**
	 * Sets information about requirement of showing grid on graphs
	 * 
	 * @param isShowGrid		true to show grid, false otherwise
	 */
	public void setShowGrid(boolean isShowGrid);
	
	
	/**
	 * Gives ImageIcon object that represents icon associated with given name
	 * 
	 * @param name	name of icon
	 * 
	 * @return		icon associated with given name	
	 */
	public ImageIcon getIcon(String name);
	
	
	/**
	 * Returns distance between nodes on ARD graph
	 * 
	 * @return distance between nodes on ARD graph
	 */
	public int getARDHorizontalNodesDistance();
	
	
	/**
	 * Sets distance between nodes on ARD graph
	 * 
	 * @param nodesDistance	new distance between nodes on ARD graph
	 */
	public void setARDHorizontalNodesDistance(int nodesDistance);
	
	/**
	 * Returns vertical distance between nodes on ARD graph.
	 * 
	 * @return	vertical distance between nodes on ARD graph
	 */
	public int getARDVerticalNodesDistance();
	
	/**
	 * Sets vertical distance between nodes on ARD graph.
	 * 
	 * @param nodesDistance		vertical distance between nodes on ARD graph.
	 */
	public void setARDVerticalNodesDistance(int nodesDistance);
	
	/**
	 * Returns distance between nodes on TPH graph
	 * 
	 * @return distance between nodes on TPH graph
	 */
	public int getTPHHorizontalNodesDistance();
	
	
	/**
	 * Sets distance between nodes on TPH graph
	 * 
	 * @param nodesDistance	new distance between nodes on TPH graph
	 */
	public void setTPHHorizontalNodesDistance(int nodesDistance);
	
	/**
	 * Returns vertical distance between nodes on TPH graph.
	 * 
	 * @return	vertical distance between nodes on TPH graph.
	 */
	public int getTPHVerticalNodesDistance();
	
	/**
	 * Sets vertical distance between nodes on TPH graph.
	 * 
	 * @param nodesDistance		vertical distance between nodes on TPH graph.
	 */
	public void setTPHVerticalNodesDistance(int nodesDistance);
	
	/**
	 * Sets flag that indicates if graphs sizes should be optimized.
	 * 
	 * @param optimize		true if graphs size skould be optimized, false otherwise
	 */
	public void setOptimizeGraphsSizes(boolean optimize);
	
	/**
	 * Indicates if graphs sizes should be optimized.
	 * 
	 * @return		true if graphs sizes should be optimized, false otherwise.
	 */
	public boolean isOptimizeGraphsSizes();
}