/*
 *     Implementation by Szymon Ksiazek <szymonh20@poczta.fm>
 *
 *     Copyright (C) 2006-9 by the HeKatE Project
 *
 *     HJEd has been develped within the HeKatE Project, 
 *     see http://hekate.ia.agh.edu.pl
 *
 *     This file is part of HJEd.
 *
 *     HJEd is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     HJEd is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with HJEd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package pl.edu.agh.hekate.ard;

/**
 * Dependency class
 * 
 * @author Szymon Książek
 *
 */
public class Dependency 
{
	/**
	 * Independent property
	 */
	private Property independent = null;
	
	/**
	 * Dependent property
	 */
	private Property dependent = null;
	
	
	/**
	 * Constructor
	 * 
	 * @param independent	independent property of dependency
	 * @param dependent		dependent property of dependency
	 */
	public Dependency(Property independent, Property dependent)
	{
		this.independent = independent;
		this.dependent = dependent;
	}
	
	
	/**
	 * Returns independent property of dependency
	 * 
	 * @return independent property of dependency
	 */
	public Property getIndependent() 
	{
		return independent;
	}

	/**
	 * Sets independent property of dependency
	 * 
	 * @param independent new independent property of dependency
	 */
	public void setIndependent(Property independent) 
	{
		this.independent = independent;
	}

	/**
	 * Returns dependent property of dependency
	 * 
	 * @return dependent property of dependency
	 */
	public Property getDependent() 
	{
		return dependent;
	}

	/**
	 * Sets dependent property of dependency
	 * 
	 * @param dependent new dependent property of dependency
	 */
	public void setDependent(Property dependent) 
	{
		this.dependent = dependent;
	}
}
