/*
 *     Implementation by Szymon Ksiazek <szymonh20@poczta.fm>
 *
 *     Copyright (C) 2006-9 by the HeKatE Project
 *
 *     HJEd has been develped within the HeKatE Project, 
 *     see http://hekate.ia.agh.edu.pl
 *
 *     This file is part of HJEd.
 *
 *     HJEd is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     HJEd is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with HJEd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package pl.edu.agh.hekate.ard;

/**
 * Interface that defines operations used for saving and opening graphs.
 *  
 * @author Szymon Książek
 *
 */
public interface PersistenceManager 
{
	/**
	 * Loads graphs from file.
	 */
	public void open();
	
	
	/**
	 * Saves graphs to selected file.
	 */
	public void saveAs();
	
	
	/**
	 * Saves graphs to the actual file.
	 */
	public void save();
	
	
	/**
	 * Indicates if actual design phase have been saved.
	 * 
	 * @return	true if actual design phase have been saved. false othervise.
	 */
	public boolean isStateSaved();
	
	
	/**
	 * Sets save status flag.
	 * 
	 * @param isStateSaved	flag indicating if actual design phase have been saved.
	 */
	public void setStateSaved(boolean isStateSaved);
	
	
	/**
	 * Sets autosave flag.
	 * 
	 * @param isAutosave	true to enable autosave mode. false otherwise.
	 */
	public void setAutosave(boolean isAutosave);
	
	
	/**
	 * Indicates if autosave mode is enabled.
	 * 
	 * @return
	 */
	public boolean isAutosave();
	
	
	/**
	 * Sets local reference to view
	 * 
	 * @param view	view reference
	 */
	public void setView(ARDView view);
}
