/*
 *     Implementation by Szymon Ksiazek <szymonh20@poczta.fm>
 *
 *     Copyright (C) 2006-9 by the HeKatE Project
 *
 *     HJEd has been develped within the HeKatE Project, 
 *     see http://hekate.ia.agh.edu.pl
 *
 *     This file is part of HJEd.
 *
 *     HJEd is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     HJEd is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with HJEd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package pl.edu.agh.hekate.ard;

import java.util.*;

/**
 * Class that represents property
 * 
 * @author Szymon Książek
 *
 */
public class Property 
{
	/**
	 * list of property attributes
	 */
	private List<Attribute> attributes = null;

	/**
	 * Constructor of property
	 * 
	 * @param attributes list of property attributes
	 */
	public Property(List<Attribute> attributes)
	{
		this.attributes = attributes;
	}
	
	
	/**
	 * Returns list of property attributes names
	 * 
	 * @return	list of property attributes names
	 */
	public List<String> getAttributesNames()
	{
		List<String> attributesNames = new ArrayList<String>();
		
		for (Attribute attribute : attributes)
		{
			attributesNames.add(attribute.getName());
		}
		
		return attributesNames;
	}
	
	
	/**
	 * Returns list of property attributes
	 * 
	 * @return	list of property attributes
	 */
	public List<Attribute> getAttributes() 
	{
		return attributes;
	}

	
	/**
	 * Sets property attributes
	 * 
	 * @param 	list of new property attributes
	 */
	public void setAttributes(List<Attribute> attributes) 
	{
		this.attributes = attributes;
	}
}
