/*
 *     Implementation by Szymon Ksiazek <szymonh20@poczta.fm>
 *
 *     Copyright (C) 2006-9 by the HeKatE Project
 *
 *     HJEd has been develped within the HeKatE Project, 
 *     see http://hekate.ia.agh.edu.pl
 *
 *     This file is part of HJEd.
 *
 *     HJEd is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     HJEd is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with HJEd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package pl.edu.agh.hekate.ard;

import java.awt.event.*;

import javax.swing.*;
import java.awt.*;
import java.util.List;
import org.jgraph.JGraph;
import pl.edu.agh.hekate.ard.actions.*;


/**
 * Mouse event handler for TPH graph.
 * 
 * @author Szymon Książek
 *
 */
public class TPHMouseListener extends MouseAdapter 
{
	/**
	 * Model instance
	 */
	private ARDModel model = null;
	
	/**
	 * View instance
	 */
	private ARDView view = null;
	
	/**
	 * Last coordinates of dragged point
	 */
	private Point dragStartPoint = null;
	
	
	/**
	 * Constructor 
	 * 
	 * @param model model instance
	 * @param view  view instance
	 */
	public TPHMouseListener(ARDModel model, ARDView view)
	{
		this.model = model;
		this.view = view;
	}
	
	/**
	 * Invoked when a mouse button has been pressed on TPH graph 
	 */
	public void mousePressed(MouseEvent e)
	{
		if (e.getButton() == MouseEvent.BUTTON1 && e.getClickCount() == 2 && !e.isAltDown())
		{
			GraphNode selectedGraphNode = null;
			
			Component component = e.getComponent();
			
			JGraph TPHGraph = (JGraph)component;
		
			Object cell = TPHGraph.getFirstCellForLocation(e.getX(), e.getY());
			
			if (cell != null && cell instanceof GraphNode)
			{
				selectedGraphNode = (GraphNode)cell;
				
				view.markAllDown(selectedGraphNode.getProperty());
				
				if (e.getClickCount() == 2)
				{
					// roll ADR graph to design phase that contain selected property 
					List<Property> transformDestinies = model.getTransformDestinies(selectedGraphNode.getProperty());
					
					if (transformDestinies.size() > 0)
					{
						Property property = transformDestinies.get(0);
						
						view.unrollAll();
						
						view.rollItem(new GraphNode(property));
					}
				}
			}
			else
			{
				view.markAllDown(null);
			}
		}
		else if (e.isAltDown())
		{
			dragStartPoint = e.getLocationOnScreen();
		}
		
		if (e.getButton() == MouseEvent.BUTTON3)
		{
			boolean showAddAttributeItem = false;
			
			GraphNode selectedGraphNode = null;
			
			if (view.getTPHGraphNodes().size() != 0)
			{
				Component component = e.getComponent();
				
				JGraph TPHGraph = (JGraph)component;
			
				Object cell = TPHGraph.getFirstCellForLocation(e.getX(), e.getY());
				
				if (cell != null && cell instanceof GraphNode)
				{
					selectedGraphNode = (GraphNode)cell;
					
					// attributes can be added only to leafs on TPH graph
					if (model.getTransformDestinies(selectedGraphNode.getProperty()).size() == 0)
					{
						view.markAllDown(selectedGraphNode.getProperty());
						
						showAddAttributeItem = true;
					}
				}
				else
				{
					view.markAllDown(null);
				}
			}
			
			JPopupMenu popupMenu = new JPopupMenu();
			
			if (showAddAttributeItem)
			{
				JMenuItem addAttributeItem = new JMenuItem("Add attribute");
				addAttributeItem.setMnemonic('A');
				addAttributeItem.addActionListener(new AddAttributeAction(model, view, selectedGraphNode.getProperty()));
				addAttributeItem.setIcon(view.getArranger().getIcon("add_attribute"));
				popupMenu.add(addAttributeItem);
			}
			
			if (popupMenu.getPopupMenuListeners().length > 0)
			{
				popupMenu.show(e.getComponent(), e.getX(), e.getY());
			}
		}
	}
	
	// invoked when TPH graph is dragged
	public void mouseDragged(MouseEvent e)
	{
		if (e.isAltDown())
		{
			Point actualPoint = e.getLocationOnScreen();
			
			int xMove = (int)(actualPoint.getX() - dragStartPoint.getX());
			int yMove = (int)(actualPoint.getY() - dragStartPoint.getY());
			
			if (xMove != 0 || yMove != 0)
			{
				view.dragTPHGraph(xMove, yMove);
			}
			
			dragStartPoint = actualPoint;
		}
	}
	
	
	public void mouseWheelMoved(MouseWheelEvent e)
	{
		// number of 'clicks' - how fast wheel is rotated
		int clicks = e.getWheelRotation();
		
		// zoom out TPH graph
		if (clicks > 0)
		{
			for (int zoomCount = 0 ; zoomCount < clicks ; zoomCount++)
			{
				view.zoomTPHGraph(false);
			}
		}
		// zoom out TPH graph
		else if (clicks < 0)
		{
			clicks = -clicks;
			
			for (int zoomCount = 0 ; zoomCount < clicks ; zoomCount++)
			{
				view.zoomTPHGraph(true);
			}
		}
	}
}