/*
 *     Implementation by Szymon Ksiazek <szymonh20@poczta.fm>
 *
 *     Copyright (C) 2006-9 by the HeKatE Project
 *
 *     HJEd has been develped within the HeKatE Project, 
 *     see http://hekate.ia.agh.edu.pl
 *
 *     This file is part of HJEd.
 *
 *     HJEd is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     HJEd is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with HJEd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package pl.edu.agh.hekate.ard.actions;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;

import pl.edu.agh.hekate.ard.*;
import pl.edu.agh.hekate.ard.dialogs.*;


/**
 * Action that is responsible for adding attribute to property
 * 
 * @author Szymon Książek
 *
 */
public class AddAttributeAction  extends AbstractAction
{
	private static final long serialVersionUID = 1L;
	
	/**
	 * Model instance
	 */
	private ARDModel model = null;
	
	/**
	 * View instance
	 */
	private ARDView view = null;
	
	/**
	 * Property that attribute is added to
	 */
	private Property property = null;
	
	/**
	 * Constructor
	 * 
	 * @param model		model instance
	 * @param view		view instance
	 * @param property	property that attribute is added to
	 */
	public AddAttributeAction(ARDModel model, ARDView view, Property property)
	{
		this.model = model;
		this.view = view;
		this.property = property;
	}
	
	public void actionPerformed(ActionEvent event)
	{
		// show add attribute dialog
		AddAttributeDialog addAttributeDialog = new AddAttributeDialog(model);
	
		if (addAttributeDialog.showDialog("Add attribute"))
		{
			// get entered attribute name
			String newAttributeName = addAttributeDialog.getAttributeName();
			
			model.addAttributeToProperty(property, newAttributeName);
			
			view.refreshGraphs();
			
			view.getPersistenceManager().setStateSaved(false);
		}
		
		view.markAllDown(null);
	}
}