/*
 *     Implementation by Szymon Ksiazek <szymonh20@poczta.fm>
 *
 *     Copyright (C) 2006-9 by the HeKatE Project
 *
 *     HJEd has been develped within the HeKatE Project, 
 *     see http://hekate.ia.agh.edu.pl
 *
 *     This file is part of HJEd.
 *
 *     HJEd is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     HJEd is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with HJEd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package pl.edu.agh.hekate.ard.actions;

import java.awt.event.ActionEvent;
import java.util.List;

import javax.swing.AbstractAction;

import pl.edu.agh.hekate.ard.*;
import pl.edu.agh.hekate.ard.dialogs.*;


/**
 * Action that is responsible for finalize property
 * 
 * @author Szymon Książek
 *
 */
public class AddFinalizationAction  extends AbstractAction
{
	private static final long serialVersionUID = 1L;
	
	/**
	 * Model instance
	 */
	private ARDModel model = null;
	
	/**
	 * View instance
	 */
	private ARDView view = null;
	
	/**
	 * graph node that represents finalized property
	 */
	private GraphNode graphNode = null;
	
	/**
	 * Constructor
	 * 
	 * @param model 		model inctance
	 * @param view			view instance
	 * @param graphNode		graph node that represents finalized property
	 */
	public AddFinalizationAction(ARDModel model, ARDView view, GraphNode graphNode)
	{
		this.model = model;
		this.view = view;
		this.graphNode = graphNode;
	}
	
	public void actionPerformed(ActionEvent event)
	{
		// show dialog
		AddFinalizationDialog addFinalizationDialog = new AddFinalizationDialog(model, graphNode);
		
		if (addFinalizationDialog.showDialog("Finalization"))
		{
			// get new attributes names from dialog
			List<String> newAttributesNames = addFinalizationDialog.getNewAttributesNames();
			
			// add finalization between given property and property represented by given attributes names
			Property newProperty = model.addFinalization(graphNode.getProperty(), newAttributesNames);
			
			// remove from ARD graph node that represents finalized property
			view.removeARDGraphNode(graphNode);
			
			// add to ARD graph node that represents property created by finalization
			view.addARDGraphNode(new GraphNode(newProperty));
			
			view.refreshGraphs();
			view.refreshARDGraph();
			
			view.getPersistenceManager().setStateSaved(false);
		}
		else
		{
			view.markAllDown(null);
		}
	}
}