/*
 *     Implementation by Szymon Ksiazek <szymonh20@poczta.fm>
 *
 *     Copyright (C) 2006-9 by the HeKatE Project
 *
 *     HJEd has been develped within the HeKatE Project, 
 *     see http://hekate.ia.agh.edu.pl
 *
 *     This file is part of HJEd.
 *
 *     HJEd is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     HJEd is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with HJEd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package pl.edu.agh.hekate.ard.actions;

import java.awt.event.ActionEvent;
import java.util.List;

import javax.swing.AbstractAction;

import pl.edu.agh.hekate.ard.*;
import pl.edu.agh.hekate.ard.dialogs.*;

/**
 * Action that is responsible for create split
 * 
 * @author Szymon Książek
 *
 */
public class AddSplitAction  extends AbstractAction
{
	private static final long serialVersionUID = 1L;
	
	/**
	 * Model instance
	 */
	private ARDModel model = null;
	
	/**
	 * View instance
	 */
	private ARDView view = null;
	
	/**
	 * Graph node that represents property that split is based on
	 */
	private GraphNode graphNode = null;
	
	
	/**
	 * Constructor
	 * 
	 * @param model		model instance
	 * @param view		view instance
	 * @param graphNode	graph node that represents property that split is based on
	 */
	public AddSplitAction(ARDModel model, ARDView view, GraphNode graphNode)
	{
		this.model = model;
		this.view = view;
		this.graphNode = graphNode;
	}
	
	public void actionPerformed(ActionEvent event)
	{
		// show dialog
		AddSplitDialog addSplitDialog = new AddSplitDialog(model, view.getArranger(), graphNode);
		
		if (addSplitDialog.showDialog("Split"))
		{
			// get list of properties created by split operation
			List<Property> newProperties = addSplitDialog.getNewProperties();
			
			// For simplicity, split dialog creates new attribute for all created properties.
			// But for split all attributes that new properties contain already exists in model.
			// We must replace every new created attribute with attribute from model with the same name.
			for (Property property : newProperties)
			{
				for (Attribute attribute : property.getAttributes())
				{
					for (Attribute modelAttribute : model.getAttributesList())
					{
						if (attribute.getName().trim().equals(modelAttribute.getName().trim()))
						{
							property.getAttributes().set(property.getAttributes().indexOf(attribute), modelAttribute);
						}
					}
				}
			}
			
			// get from dialog list that contains dependencies between created properties
			List<Dependency> newDependencies = addSplitDialog.getNewDependenciesList();
			
			// add split to model
			model.addSplit(graphNode.getProperty(), newProperties, newDependencies);
			
			// remove from ARD graph node that represents old property
			view.removeARDGraphNode(graphNode);
			
			// add to ARD graph nodes that represent new properties 
			for (Property newProperty : newProperties)
			{
				view.addARDGraphNode(new GraphNode(newProperty));
			}
			
			view.refreshGraphs();
			view.refreshARDGraph();
			
			view.getPersistenceManager().setStateSaved(false);
		}
		else
		{
			view.markAllDown(null);
		}
	}
}