/*
 *     Implementation by Szymon Ksiazek <szymonh20@poczta.fm>
 *
 *     Copyright (C) 2006-9 by the HeKatE Project
 *
 *     HJEd has been develped within the HeKatE Project, 
 *     see http://hekate.ia.agh.edu.pl
 *
 *     This file is part of HJEd.
 *
 *     HJEd is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     HJEd is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with HJEd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package pl.edu.agh.hekate.ard.actions;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Icon;

import pl.edu.agh.hekate.ard.*;

/**
 * Action that is responsible for change distance between graph nodes
 * 
 * @author Szymon Książek
 *
 */
public class ChangeNodesDistanceAction extends AbstractAction
{
	private static final long serialVersionUID = 1L;
	
	/**
	 * view instance
	 */
	private ARDView view = null;
	
	/**
	 * graph type ('ARD' to ARD graph, 'TPH' to TPH graph)
	 */
	private String graphType = null;
	
	/**
	 * true to increase nodes distance, false to decrease nodes distance
	 */
	private boolean increaseDistance = false;
	
	private String distanceType = "";
	
	
	/**
	 * Constructor
	 * 
	 * @param name				name of action
	 * @param icon				icon of action
	 * @param graphType			graph type ('ARD' to ARD graph, 'TPH' to TPH graph)
	 * @param distanceType		'horizontal' for horizontal distance, 'vertical' for vertical distance
	 * @param increaseDistance	true for increase fistance, false for decrease distance
	 * @param view				view reference
	 */
	public ChangeNodesDistanceAction(String name, Icon icon, String graphType, String distanceType, boolean increaseDistance, ARDView view)
	{
		this.graphType = graphType;
		this.increaseDistance = increaseDistance;
		this.distanceType = distanceType;
		this.view = view;
		
		putValue(Action.NAME, name);
		putValue(Action.SMALL_ICON, icon);
		putValue(Action.SHORT_DESCRIPTION, name);
	}
	
	public void actionPerformed(ActionEvent event)
	{
		if (graphType.equals("ARD"))
		{
			if (distanceType.equals("horizontal"))
			{
				view.changeARDNodesDistance(increaseDistance, true);
			}
			else
			{
				view.changeARDNodesDistance(increaseDistance, false);
			}
			
			view.refreshARDGraph();
		}
		if (graphType.equals("TPH"))
		{
			if (distanceType.equals("horizontal"))
			{
				view.changeTPHNodesDistance(increaseDistance, true);
			}
			else
			{
				view.changeTPHNodesDistance(increaseDistance, false);
			}
			
			view.refreshTPHGraph();
		}
	}
}