/*
 *     Implementation by Szymon Ksiazek <szymonh20@poczta.fm>
 *
 *     Copyright (C) 2006-9 by the HeKatE Project
 *
 *     HJEd has been develped within the HeKatE Project, 
 *     see http://hekate.ia.agh.edu.pl
 *
 *     This file is part of HJEd.
 *
 *     HJEd is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     HJEd is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with HJEd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package pl.edu.agh.hekate.ard.actions;

import java.awt.event.ActionEvent;

import javax.swing.*;

import pl.edu.agh.hekate.ard.*;

/**
 * Action that is responsible from exit from application
 * 
 * @author Szymon Książek
 *
 */
public class ExitAction extends AbstractAction
{
	private static final long serialVersionUID = 1L;

	/**
	 * Reference to instance of persistence manager
	 */
	private PersistenceManager persistenceManager = null;
	
	/**
	 * Reference to main frame of application
	 */
	private MainFrame mainFrame = null;
	
	
	/**
	 * Constructor
	 * 
	 * @param name					name of action
	 * @param icon					icon of action
	 * @param persistenceManager	reference to instance of persistence manager	
	 * @param mainFrame				reference to main frame of application
	 */
	public ExitAction(String name, Icon icon, PersistenceManager persistenceManager, MainFrame mainFrame)
	{
		this.persistenceManager = persistenceManager;
		this.mainFrame = mainFrame;
		
		putValue(Action.NAME, name);
		putValue(Action.SMALL_ICON, icon);
		putValue(Action.SHORT_DESCRIPTION, name);
	}
	
	public void actionPerformed(ActionEvent event)
	{
		boolean stateSaved = true;
		
		// while actual state is not saved
		while (!persistenceManager.isStateSaved())
		{
			stateSaved = false;
			
			Object[] options = {"Yes", "No", "Cancel"};
			
			int n = JOptionPane.showOptionDialog(
					mainFrame, 
					"The model have been modified. Save changes?", 
					"Warning",
					JOptionPane.YES_NO_CANCEL_OPTION,
					JOptionPane.QUESTION_MESSAGE,
					null,
					options,
					options[2]);
			
			// if user choose "Yes", save state
			if (n == 0)
			{
				persistenceManager.save();
			}
			
			// if user choose "No", don't show dialog again and exit from application without save 
			if (n == 1)
			{
				break;
			}
			
			// if user choose "Cancel" option, interrupt action and back to application
			if (n == 2)
			{
				return;
			}
		}
			
		// is actual state is saved, show confirm dialog
		if (stateSaved)
		{
			Object[] options = {"Yes", "No"};
			
			int n = JOptionPane.showOptionDialog(
					mainFrame, 
					"Are you sure you want to quit?", 
					"Warning",
					JOptionPane.YES_NO_OPTION,
					JOptionPane.QUESTION_MESSAGE,
					null,
					options,
					options[1]);
			
			if (n == 0)
			{
				mainFrame.dispose();
				System.exit(0);
			}
			if (n == 1)
			{
				return;
			}
		}
		else
		{
			
			mainFrame.dispose();
			
			System.exit(0);
		}
	}
}