/*
 *     Implementation by Szymon Ksiazek <szymonh20@poczta.fm>
 *
 *     Copyright (C) 2006-9 by the HeKatE Project
 *
 *     HJEd has been develped within the HeKatE Project, 
 *     see http://hekate.ia.agh.edu.pl
 *
 *     This file is part of HJEd.
 *
 *     HJEd is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     HJEd is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with HJEd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package pl.edu.agh.hekate.ard.actions;

import java.awt.event.ActionEvent;

import java.util.*;

import javax.swing.*;

import pl.edu.agh.hekate.ard.*;

/**
 * Action that is responsible for create new (empty) graphs
 * 
 * @author Szymon Książek
 *
 */
public class NewAction extends AbstractAction
{
	private static final long serialVersionUID = 1L;
	
	/**
	 * model instance
	 */
	private ARDModel model = null;
	
	/**
	 * view instance
	 */
	private ARDView view = null;
	
	/**
	 * persistence manager instance
	 */
	private PersistenceManager persistenceManager = null;
	
	/**
	 * main frame instance
	 */
	MainFrame mainFrame = null;
	
	
	/**
	 * Constructor
	 * 
	 * @param name					name of action
	 * @param icon					icon of action
	 * @param model					model instance
	 * @param view					view instance
	 * @param persistenceManager	persistence manager instance
	 */
	public NewAction(String name, Icon icon, ARDModel model, ARDView view, PersistenceManager persistenceManager, MainFrame mainFrame)
	{
		this.model = model;
		this.view = view;
		this.persistenceManager = persistenceManager;
		this.mainFrame = mainFrame;
		
		putValue(Action.NAME, name);
		putValue(Action.SMALL_ICON, icon);
		putValue(Action.SHORT_DESCRIPTION, name);
	}
	
	public void actionPerformed(ActionEvent event)
	{
		// before create new graphs, ask user if he want to save old graphs
		while (!persistenceManager.isStateSaved())
		{
			Object[] options = {"Yes", "No", "Cancel"};
			
			int n = JOptionPane.showOptionDialog(
					null, 
					"The model have been modified. Save changes?", 
					"Warning",
					JOptionPane.YES_NO_CANCEL_OPTION,
					JOptionPane.QUESTION_MESSAGE,
					null,
					options,
					options[2]);
			
			if (n == 0)
			{
				persistenceManager.save();
			}
			if (n == 1)
			{
				break;
			}
			if (n == 2)
			{
				return;
			}
		}
		
		// clear model data
		model.setAttributesList(new ArrayList<Attribute>());
		model.setPropertiesList(new ArrayList<Property>());
		model.setDependenciesList(new ArrayList<Dependency>());
		model.setTransformsList(new ArrayList<Transform>());
		
		view.setARDGraphNodes(new ArrayList<GraphNode>());
		
		view.setARDScale(1.0);
		view.setTPHScale(1.0);
		
		persistenceManager.setStateSaved(false);
		persistenceManager.setStateSaved(true);
		
		mainFrame.setTitle("ARD Editor");
		
		view.refreshGraphs();
	}
}