/*
 *     Implementation by Szymon Ksiazek <szymonh20@poczta.fm>
 *
 *     Copyright (C) 2006-9 by the HeKatE Project
 *
 *     HJEd has been develped within the HeKatE Project, 
 *     see http://hekate.ia.agh.edu.pl
 *
 *     This file is part of HJEd.
 *
 *     HJEd is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     HJEd is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with HJEd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package pl.edu.agh.hekate.ard.actions;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Icon;

import pl.edu.agh.hekate.ard.*;

/**
 * Action that is responsible for optimize graph's size
 * 
 * @author Szymon Książek
 *
 */
public class OptimizeDiagramSizeAction extends AbstractAction
{
	private static final long serialVersionUID = 1L;
	
	/**
	 * view instance
	 */
	private ARDView view = null;
	
	/**
	 * graph type ('ARD' to ARD graph, 'TPH' to TPH graph)
	 */
	private String graphType = null;
	
	
	/**
	 * Constructor
	 * 
	 * @param name			name of action
	 * @param icon			icon of action
	 * @param graphType		graph type ('ARD' to ARD graph, 'TPH' to TPH graph)
	 * @param zoomIn		true to increase nodes distance, false to decrease nodes distance
	 * @param view			view instance
	 */
	public OptimizeDiagramSizeAction(String name, Icon icon, String graphType, ARDView view)
	{
		this.graphType = graphType;
		this.view = view;
		
		putValue(Action.NAME, name);
		putValue(Action.SMALL_ICON, icon);
		putValue(Action.SHORT_DESCRIPTION, name);
	}
	
	public void actionPerformed(ActionEvent event)
	{
		if (graphType.equals("ARD"))
		{
			view.optimizeARDSize();
		}
		if (graphType.equals("TPH"))
		{
			view.optimizeTPHSize();
		}
	}
}