/*
 *     Implementation by Szymon Ksiazek <szymonh20@poczta.fm>
 *
 *     Copyright (C) 2006-9 by the HeKatE Project
 *
 *     HJEd has been develped within the HeKatE Project, 
 *     see http://hekate.ia.agh.edu.pl
 *
 *     This file is part of HJEd.
 *
 *     HJEd is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     HJEd is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with HJEd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package pl.edu.agh.hekate.ard.actions;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;

import pl.edu.agh.hekate.ard.ARDView;
import pl.edu.agh.hekate.ard.GraphNode;

/**
 * Action that is responsible for roll/unroll graph item
 * 
 * @author Szymon Książek
 *
 */
public class RollAction extends AbstractAction
{
	private static final long serialVersionUID = 1L;
	
	/**
	 * view instance
	 */
	private ARDView view = null;
	
	/**
	 * graph node that is rolled/unrolled by this action
	 * 
	 */
	private GraphNode graphNode = null;
	
	/**
	 * true to roll graph node, false to unroll graph node
	 */
	private boolean roll = false;
	
	
	/**
	 * Constructor
	 * 
	 * @param view		view instance
	 * @param graphNode	graph node that is rolled/unrolled
	 * @param roll		true to roll graph node, false to unroll
	 */
	public RollAction(ARDView view, GraphNode graphNode, boolean roll)
	{
		this.view = view;
		this.graphNode = graphNode;
		this.roll = roll;
	}
	
	public void actionPerformed(ActionEvent event)
	{
		if (roll)
		{
			view.rollItem(graphNode);
		}
		else
		{
			view.unrollItem(graphNode);
		}
	}
}