/*
 *     Implementation by Szymon Ksiazek <szymonh20@poczta.fm>
 *
 *     Copyright (C) 2006-9 by the HeKatE Project
 *
 *     HJEd has been develped within the HeKatE Project, 
 *     see http://hekate.ia.agh.edu.pl
 *
 *     This file is part of HJEd.
 *
 *     HJEd is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     HJEd is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with HJEd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package pl.edu.agh.hekate.ard.dialogs;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.util.*;

import javax.swing.*;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import pl.edu.agh.hekate.ard.ARDModel;

import pl.edu.agh.hekate.ard.*;


/**
 * Dialog used to create finalization
 * 
 * @author Szymon Książek
 *
 */
public class AddFinalizationDialog extends JPanel
{
	private static final long serialVersionUID = 1L;
	
	/**
	 * model reference
	 */
	private ARDModel model;
	
	/**
	 * field that is used to enter new attribute name
	 */
	private JTextField newAttributeNameField;
	
	/**
	 * button used to add entered attribute name
	 */
	private JButton addAttributeButton;
	
	/**
	 * list of attributes created by finalization
	 */
	private JList newAttributesList;
	
	/**
	 * button used to remove selected attributes
	 */
	private JButton removeAttributesButton;
	
	/**
	 * error message presented to user
	 */
	private String errorMessage = "";
	
	/**
	 * label that contains error message
	 */
	private JLabel errorLabel;
	
	/**
	 * list that contains attributes names entered by user
	 */
	private List<String> newAttributesNamesList;
	
	/**
	 * true if user want to close window and confirm finalization
	 */
	private boolean ok;
	
	/**
	 * dialog reference
	 */
	private JDialog dialog;
	
	
	/**
	 * Constructor
	 * 
	 * @param model			model reference
	 * @param graphNode		graph node that is finalized
	 */
	public AddFinalizationDialog(ARDModel model, GraphNode graphNode)
	{
		this.model = model;
		
		setLayout(new BorderLayout());
		
		JPanel upperPanel = new JPanel();
		
		upperPanel.setLayout(new FlowLayout());
	
		// show name of finalized attribute
		upperPanel.add(new JLabel("Finalized attribute: "));
	
		upperPanel.add(new JLabel(graphNode.getProperty().getAttributes().get(0).getName()));
		
		add(upperPanel, BorderLayout.NORTH);
		
		
		JPanel centralPanel = new JPanel();
		centralPanel.setLayout(new BorderLayout());
		
		JPanel addAttributePanel = new JPanel();
		
		JPanel addAttributeLabelPanel = new JPanel();
		addAttributeLabelPanel.setLayout(new GridLayout(2, 1, 10, 5));
		
		addAttributeLabelPanel.add(new JLabel("New attribute: "));
		addAttributeLabelPanel.add(new JLabel(""));
		
		addAttributePanel.add(addAttributeLabelPanel);
		
		JPanel addAttributeNamePanel = new JPanel();
		addAttributeNamePanel.setLayout(new GridLayout(2, 1, 10, 5));
		
		newAttributeNameField = new JTextField(20);
		
		addAttributeNamePanel.add(newAttributeNameField);
		
		errorLabel = new JLabel(errorMessage);
		
		errorLabel.setForeground(Color.RED);
		
		addAttributeNamePanel.add(errorLabel);
		
		addAttributePanel.add(addAttributeNamePanel);
		
		JPanel addAttributeButtonPanel = new JPanel();
		
		addAttributeButtonPanel.setLayout(new GridLayout(2, 1, 10, 5));
		
		
		// button used to add new attribute to attributes list
		addAttributeButton = new JButton("Add");
		
		addAttributeButton.addActionListener(new
				ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				addNewAttribute();
			}
		});
		
		addAttributeButtonPanel.add(addAttributeButton);
		addAttributeButtonPanel.add(new JLabel(""));
		
		addAttributePanel.add(addAttributeButtonPanel);
		
		centralPanel.add(addAttributePanel, BorderLayout.NORTH);
		
		
		JPanel removeAttributesPanel = new JPanel();
		
		
		// button used to remove attributes from new attributes list
		removeAttributesButton = new JButton("Remove");
		
		removeAttributesButton.addActionListener(new
				ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				removeSelectedAttributes();
			}
		});
		
		removeAttributesPanel.add(removeAttributesButton);
		
		
		centralPanel.add(removeAttributesPanel, BorderLayout.SOUTH);
		
		
		newAttributesList = new JList();
		
		JScrollPane attributesListScrollPane = new JScrollPane(newAttributesList);
		
		attributesListScrollPane.setBorder(BorderFactory.createEmptyBorder(0, 50, 0, 50));
		
		centralPanel.add(attributesListScrollPane);
		
		centralPanel.setBorder(new javax.swing.border.TitledBorder("New attributes"));
		
		add(centralPanel, BorderLayout.CENTER);
		
		
		// create buttons panel 
		JPanel buttonsPanel = new JPanel();
		
		JButton OKButton = new JButton("OK");
		
		OKButton.addActionListener(new
			ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					submitDialog();
				}
			});
		
		
		JButton cancelButton = new JButton("Cancel");
		
		cancelButton.addActionListener(new
			ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					ok = false;
					dialog.setVisible(false);
				}
			});
		
		
		buttonsPanel.add(OKButton);
		buttonsPanel.add(cancelButton);
		
		add(buttonsPanel, BorderLayout.SOUTH);
		
		
		this.setBorder(BorderFactory.createEmptyBorder(20, 50, 15, 50));
		
		newAttributeNameField.addActionListener(new
				ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				addAttributeButton.doClick();
			}
		});
		
		newAttributesNamesList = new ArrayList<String>();
	}
	
	
	/**
	 * Method used to add new attribute to attributes list
	 */
	private void addNewAttribute()
	{
		// get entered attribute name
		String newAttributeName = newAttributeNameField.getText().trim();
		
		boolean isCorrect = true;		// if entered attribute name is valid
		
		errorMessage = "";
		
		// entered attribute name can not be empty string
		if (newAttributeName.equals(""))
		{
			isCorrect = false;
			errorMessage = "Type attribute name";
		}
		
		// entered attribute name can not be name of some existing attribute
		if (isCorrect)
		{
			Attribute attribute = model.getAttribute(newAttributeName.trim());
			
			if (attribute != null)
			{
				isCorrect = false;
				errorMessage = "Attribute exists";
			}
		}
		
		// the same name can not be entered more than once
		if (isCorrect)
		{
			if (newAttributesNamesList.contains(newAttributeName.trim()))
			{
				isCorrect = false;
				errorMessage = "You've added such attribute";
			}
		}
		
		// if entered attribute name is valid, add it to attributes list
		if (isCorrect)
		{
			newAttributeNameField.setText("");
			errorLabel.setText(errorMessage);
			newAttributesNamesList.add(newAttributeName);
			refreshAttributesList();
			dialog.pack();
		}
		// if entered attribute name is not valid, show error message
		else
		{
			errorLabel.setText(errorMessage);
			dialog.pack();
			
			int screenWidth = (int)Toolkit.getDefaultToolkit().getScreenSize().getWidth();
			int screenHeight = (int)Toolkit.getDefaultToolkit().getScreenSize().getHeight();
			dialog.setLocation((int)(screenWidth - dialog.getSize().getWidth()) / 2, (int)(screenHeight - dialog.getSize().getHeight()) / 2);
		}
	}
	
	
	/**
	 * Method used to remove selected attributes from attributes list
	 */
	private void removeSelectedAttributes()
	{
		// get selected attributes
		Object[] selectedItems = newAttributesList.getSelectedValues();
		
		// remove selected attributes from attributes list
		for (int i = 0 ; i < selectedItems.length ; i++)
		{
			String selectedName = (String)selectedItems[i];
			
			newAttributesNamesList.remove(selectedName.trim());
		}
		
		refreshAttributesList();
	}
	
	
	/**
	 * Method used to refresh new attributes list
	 */
	private void refreshAttributesList()
	{
		// get names of new attributes
		String[] attributesNames = new String[newAttributesNamesList.size()];
		
		for (int i = 0 ; i < newAttributesNamesList.size() ; i++)
		{
			String attributeName = (String)newAttributesNamesList.get(i);
			
			attributesNames[i] = attributeName;
		}
		
		// show list that contains actual attributes names
		newAttributesList.setListData(attributesNames);
		
		dialog.pack();
		
		validate();
		
		int screenWidth = (int)Toolkit.getDefaultToolkit().getScreenSize().getWidth();
		int screenHeight = (int)Toolkit.getDefaultToolkit().getScreenSize().getHeight();
		
		dialog.setLocation((int)(screenWidth - dialog.getSize().getWidth()) / 2, (int)(screenHeight - dialog.getSize().getHeight()) / 2);
	}
	
	
	/**
	 * Method used to submit transformation
	 */
	private void submitDialog()
	{
		// finalization can be submitted only if attributes list created by finalization is not empty
		if (newAttributesNamesList.size() > 0)
		{
			ok = true;
			dialog.setVisible(false);
		}
		else
		{
			JOptionPane.showMessageDialog(dialog, "You have to add at least one attribute", "Error", JOptionPane.ERROR_MESSAGE);
		}
	}
	
	
	public boolean showDialog(String title)
	{
		ok = false;
		
		dialog = new JDialog(new JFrame(), title, true);
		
		int screenWidth = (int)Toolkit.getDefaultToolkit().getScreenSize().getWidth();
		int screenHeight = (int)Toolkit.getDefaultToolkit().getScreenSize().getHeight();
		
		dialog.getContentPane().add(this);
		
		dialog.pack();
		
		dialog.setLocation((int)(screenWidth - dialog.getSize().getWidth()) / 2, (int)(screenHeight - dialog.getSize().getHeight()) / 2);
		
		dialog.setResizable(false);
		
		dialog.setVisible(true);
		return ok;
	}
	
	
	/**
	 * Returns list of names of new property attributes
	 * 
	 * @return	new property attributes list 
	 */
	public List<String> getNewAttributesNames()
	{
		return newAttributesNamesList;
	}
}
