/*
 *     Implementation by Szymon Ksiazek <szymonh20@poczta.fm>
 *
 *     Copyright (C) 2006-9 by the HeKatE Project
 *
 *     HJEd has been develped within the HeKatE Project, 
 *     see http://hekate.ia.agh.edu.pl
 *
 *     This file is part of HJEd.
 *
 *     HJEd is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     HJEd is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with HJEd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package pl.edu.agh.hekate.ard.dialogs;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * Dialog used to add first property to diagram
 * 
 * @author Szymon Książek
 *
 */
public class AddRootNodeDialog extends JPanel
{
	private static final long serialVersionUID = 1L;
	
	/**
	 * field used to enter attribute name
	 */
	private JTextField attributeNameField;
	
	/**
	 * true if user want to close dialog
	 */
	private boolean ok;
	
	/**
	 * dialog reference
	 */
	private JDialog dialog;
	
	/**
	 * error message presented to user
	 */
	private String errorMessage = "";
	
	/**
	 * contain error message
	 */
	private JLabel errorLabel;
	
	/**
	 * OK button reference
	 */
	private JButton OKButton;
	
	/**
	 * Cancel button reference
	 */
	private JButton cancelButton;
	
	
	/**
	 * Constructor
	 */
	public AddRootNodeDialog()
	{	
		setLayout(new BorderLayout());
		
		attributeNameField = new JTextField(20);
		
		JPanel centralPanel = new JPanel();
		centralPanel.setLayout(new FlowLayout());
		
		JPanel leftPanel = new JPanel();
		leftPanel.setLayout(new GridLayout(2, 1, 10, 5));
		leftPanel.add(new JLabel("Attribute name: "));
		centralPanel.add(new JLabel(""));
		
		JPanel rightPanel = new JPanel();
		rightPanel.setLayout(new GridLayout(2, 1, 10, 5));
		
		rightPanel.add(attributeNameField);
		
		errorLabel = new JLabel(errorMessage);
		errorLabel.setForeground(Color.RED);
		rightPanel.add(errorLabel);
		
		centralPanel.add(leftPanel);
		centralPanel.add(rightPanel);
		
		add(centralPanel, BorderLayout.CENTER);
		
		
		JPanel buttonsPanel = new JPanel();
		
		OKButton = new JButton("OK");
		
		OKButton.addActionListener(new
			ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					if (isCorrectData())
					{
						ok = true;
						dialog.setVisible(false);
					}
				}
			});
		
		
		cancelButton = new JButton("Cancel");
		
		cancelButton.addActionListener(new
			ActionListener()
			{
				public void actionPerformed(ActionEvent e)
				{
					ok = false;
					dialog.setVisible(false);
				}
			});
		
		buttonsPanel.add(OKButton);
		buttonsPanel.add(cancelButton);
		
		add(buttonsPanel, BorderLayout.SOUTH);
		
		this.setBorder(BorderFactory.createEmptyBorder(20, 30, 5, 30));
		
		attributeNameField.addActionListener(new
				ActionListener()
		{
			public void actionPerformed(ActionEvent e)
			{
				OKButton.doClick();
			}
		});
	}
	
	
	/**
	 * Method that check if entered attribute name is valid
	 * 
	 * @return true if attribute name is valid, false othervise
	 */
	private boolean isCorrectData()
	{
		boolean isCorrect = true;
		
		// attribute name can not be ampty string
		if (attributeNameField.getText().trim().equals(""))
		{
			isCorrect = false;
			errorMessage = "Type attribute name";
		}
		
		// attribute must be conceptual attribute (starts with upper case)
		if (isCorrect && attributeNameField.getText().trim().charAt(0) !=
				attributeNameField.getText().trim().toUpperCase().charAt(0))
		{
			isCorrect = false;
			errorMessage = "Attribute name must start with upper case";
		}
		
		// if attribute name is not valid, show error message
		if (!isCorrect)
		{
			errorLabel.setText(errorMessage);
			dialog.pack();
			
			int screenWidth = (int)Toolkit.getDefaultToolkit().getScreenSize().getWidth();
			int screenHeight = (int)Toolkit.getDefaultToolkit().getScreenSize().getHeight();
			dialog.setLocation((int)(screenWidth - dialog.getSize().getWidth()) / 2, (int)(screenHeight - dialog.getSize().getHeight()) / 2);
		}
		
		return isCorrect;
	}

	/**
	 * Shows dialog with given title
	 * 
	 * @param title		dialog title
	 * 
	 * @return	true if user want to confirm operation
	 */
	public boolean showDialog(String title)
	{
		ok = false;
		
		dialog = new JDialog(new JFrame(), title, true);
		
		int screenWidth = (int)Toolkit.getDefaultToolkit().getScreenSize().getWidth();
		int screenHeight = (int)Toolkit.getDefaultToolkit().getScreenSize().getHeight();
		
		dialog.getContentPane().add(this);
		
		dialog.pack();
		
		dialog.setLocation((int)(screenWidth - dialog.getSize().getWidth()) / 2, (int)(screenHeight - dialog.getSize().getHeight()) / 2);
		
		dialog.setResizable(false);
		
		dialog.setVisible(true);
		return ok;
	}
	
	/**
	 * Returns root attribute name
	 * 
	 * @return	attribute name
	 */
	public String getAttributeName() 
	{
		return attributeNameField.getText().trim();
	}
}