/*
 *     Implementation by Szymon Ksiazek <szymonh20@poczta.fm>
 *
 *     Copyright (C) 2006-9 by the HeKatE Project
 *
 *     HJEd has been develped within the HeKatE Project, 
 *     see http://hekate.ia.agh.edu.pl
 *
 *     This file is part of HJEd.
 *
 *     HJEd is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     HJEd is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with HJEd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package pl.edu.agh.hekate.ard.dialogs;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.geom.Rectangle2D;

import java.util.*;
import javax.swing.*;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;

import org.jgraph.JGraph;
import org.jgraph.graph.DefaultEdge;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.DefaultGraphModel;
import org.jgraph.graph.GraphConstants;

import com.jgraph.layout.JGraphFacade;

import pl.edu.agh.hekate.ard.ARDModel;
import pl.edu.agh.hekate.ard.GraphNode;
import pl.edu.agh.hekate.ard.Property;

import pl.edu.agh.hekate.ard.*;


/**
 * Dialog used to create split operation
 * 
 * @author Szymon Książek
 *
 */
public class AddSplitDialog extends JPanel
{
	private static final long serialVersionUID = 1L;
	
	/**
	 * model reference
	 */
	private ARDModel model = null;
	
	/**
	 * arranger reference
	 */
	private Arranger arranger = null;
	
	/**
	 * graph node that is 'splitted'
	 */
	private GraphNode graphNode;
	
	/**
	 * true if user want to close dialog and confirm operation
	 */
	private boolean ok;
	
	/**
	 * dialog reference
	 */
	private JDialog dialog;
	
	/**
	 * list of lists of new properties attributes
	 */
	private List<List<String>> newAttributesNames;
	
	/**
	 * list that contains attributes of property that is 'splitted'
	 */
	private JList oldAttributesList = null;
	
	/**
	 * list that contains attributes of selected new property
	 */
	private JList newAttributesList = null;
	
	/**
	 * number of new properties
	 */
	private int newPropertyNumber = 0;
	
	/**
	 * panel used to create split
	 */
	private JPanel createSplitPanel = null;
	
	/**
	 * panel that contains graph
	 */
	private JPanel graphPanel = null;
	
	/**
	 * graph used to customize split
	 */
	private JGraph graph = null;
	
	/**
	 * scroll pane that contains graph
	 */
	private JScrollPane graphScrollPane = null;
	
	/**
	 * nodes that are independent to 'splitted' property
	 */
	private List<GraphNode> ingoingPropertiesNodes = null;
	
	/**
	 * nodes that are dependent of 'splitted' property
	 */
	private List<GraphNode> outgoingPropertiesNodes = null;
	
	/**
	 * nodes created by split operation
	 */
	private List<GraphNode> newPropertiesNodes = null;
	
	/**
	 * panel that contains old attributes scroll pane
	 */
	private JPanel attributesLeftPanel = null;
	
	/**
	 * panel that contains new attributes scroll pane
	 */
	private JPanel attributesRightPanel = null;
	
	/**
	 * scroll pane that contains attributes names of selected property that is created by split operation
	 */
	private JScrollPane newAttributesScrollPane = null;
	
	/**
	 * scroll pane that contains attributes names of 'splitted' property
	 */
	private JScrollPane oldAttributesScrollPane = null;
	
	/**
	 * list that contains attributes of 'splitted' property
	 */
	private List<String> oldAttributesNames = null;
	
	/**
	 * list that contains edges that represents dependencies created by split operation
	 */
	private List<DefaultEdge> newEdgesList = null;
	
	/**
	 * panel that contains message that is presented to user
	 */
	private JPanel labelPanel = null;
	
	/**
	 * holds reference to selected node
	 */
	private GraphNode independentGraphNode = null;
	
	/**
	 * contains cells list used to create graph
	 */
	private List<DefaultGraphCell> cellsList = new ArrayList<DefaultGraphCell>();

	
	/**
	 * Constructor
	 * 
	 * @param model		model reference
	 * @param graphNode	graph node that is 'splitted' 
	 */
	public AddSplitDialog(ARDModel _model, Arranger _arranger, GraphNode _graphNode)
	{
		this.model = _model;
		this.arranger = _arranger;
		this.graphNode = _graphNode;
		
		setLayout(new BorderLayout());
		
		
		newAttributesNames = new ArrayList<List<String>>();
		
		newAttributesNames.add(new ArrayList<String>());
		newAttributesNames.add(new ArrayList<String>());
		
		createSplitPanel = new JPanel();
		createSplitPanel.setLayout(new BorderLayout());
		
		JPanel bottomPanel = new JPanel();
		bottomPanel.setLayout(new BorderLayout());
		
		graphPanel = new JPanel();
		graphPanel.setLayout(new BorderLayout());
		
		JPanel graphButtonsPanel = new JPanel();
		
		JButton removePropertiesButton = new JButton("Remove");
		JButton cleanEdgesButton = new JButton("Clean edges");

		// used to remove selected properties
		removePropertiesButton.addActionListener(new
				ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						removeSelected();
					}
				});
		
		// used to remove all created edges
		cleanEdgesButton.addActionListener(new
				ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						cleanEdges();
					}
				});
		
		graphButtonsPanel.add(removePropertiesButton);
		graphButtonsPanel.add(cleanEdgesButton);
		
		graphPanel.add(graphButtonsPanel, BorderLayout.SOUTH);
		
		// create list of properties that are independent to 'splitted' property 
		ingoingPropertiesNodes = new ArrayList<GraphNode>();
		
		List<Property> inProperties = model.getIndependentProperties(graphNode.getProperty());
		
		for (Property property : inProperties)
		{
			if (model.getTransformDestinies(property).size() == 0)
			{
				GraphNode independentGraphNode = new GraphNode(property);
			
				ingoingPropertiesNodes.add(independentGraphNode);
			}
		}
		
		
		// create list of properties that are dependent to 'splitted' property
		outgoingPropertiesNodes = new ArrayList<GraphNode>();
		
		List<Property> outProperties = model.getDependentProperties(graphNode.getProperty());
		
		for (Property property : outProperties)
		{
			if (model.getTransformDestinies(property).size() == 0)
			{
				GraphNode dependentGraphNode = new GraphNode(property);
			
				outgoingPropertiesNodes.add(dependentGraphNode);
			}
		}
		
		// create list of new properties (properties that will be created as a result of split operation)
		newPropertiesNodes = new ArrayList<GraphNode>();
		
		
		GraphNode newGraphNode = new GraphNode(new Property(new ArrayList<Attribute>()));
		
		newPropertiesNodes.add(newGraphNode);
		
		newGraphNode = new GraphNode(new Property(new ArrayList<Attribute>()));
		
		newPropertiesNodes.add(newGraphNode);
		
		newEdgesList = new ArrayList<DefaultEdge>();
		
		createSplitGraphScrollPane();
		
		graphPanel.add(graphScrollPane);
		
		
		// panel with attributes of selected property and attributes of properties that belongs to 'splitted' property
		JPanel attributesPanel = new JPanel();
		
		attributesLeftPanel = new JPanel();
		attributesLeftPanel.setLayout(new BorderLayout());
		
		attributesLeftPanel.add(new JLabel("Attributes:"), BorderLayout.NORTH);
		
		oldAttributesList = new JList();
		
		Property oldProperty = graphNode.getProperty();
		oldAttributesNames = oldProperty.getAttributesNames();
		
		oldAttributesList.setListData(oldAttributesNames.toArray());
		
		oldAttributesScrollPane = new JScrollPane(oldAttributesList);
		oldAttributesScrollPane.setPreferredSize(new Dimension(150,200));
		oldAttributesScrollPane.setWheelScrollingEnabled(true);
		
		attributesLeftPanel.add(oldAttributesScrollPane);
		
		attributesPanel.add(attributesLeftPanel);
		
		
		JPanel attributesCenterPanel = new JPanel();
		attributesCenterPanel.setLayout(new BorderLayout());
		
		JButton addAttributesButton = new JButton(">>");
		JButton removeAttributesButton = new JButton("<<");
		
		// used to add selected attributes to selected property
		addAttributesButton.addActionListener(new
				ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						addAttributes();
					}
				});
		
		// used to remove selected attributes from selected property
		removeAttributesButton.addActionListener(new
				ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						removeAttributes();
					}
				});
		
		attributesCenterPanel.add(addAttributesButton, BorderLayout.NORTH);
		attributesCenterPanel.add(new JLabel("   "));
		attributesCenterPanel.add(removeAttributesButton, BorderLayout.SOUTH);
		
		attributesPanel.add(attributesCenterPanel);
		
		
		attributesRightPanel = new JPanel();
		attributesRightPanel.setLayout(new BorderLayout());
		
		attributesRightPanel.add(new JLabel("New property:"), BorderLayout.NORTH);
		
		newAttributesList = new JList();
		newPropertyNumber = 0;
		
		newAttributesScrollPane = new JScrollPane(newAttributesList);
		newAttributesScrollPane.setPreferredSize(new Dimension(150,200));
		newAttributesScrollPane.setWheelScrollingEnabled(true);
		
		attributesRightPanel.add(newAttributesScrollPane);
		
		attributesPanel.add(attributesRightPanel);
		
		newPropertyNumber = -1;
		
		
		// buttons panel
		JPanel buttonsPanel = new JPanel();
		buttonsPanel.setLayout(new FlowLayout());
		
		
		// button used to cancel split operation
		JButton cancelButton = new JButton("Cancel");
		
		cancelButton.addActionListener(new
				ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						ok = false;
						dialog.setVisible(false);
					}
				});
		
		
		// button used to confirm split operation
		JButton OKButton = new JButton("OK");
		
		OKButton.addActionListener(new
				ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						if (validateSplit())
						{
							ok = true;
							dialog.setVisible(false);
						}
					}
				});
		
		buttonsPanel.add(cancelButton);
		buttonsPanel.add(OKButton);
		
		
		JPanel testPanel = new JPanel();
		testPanel.add(graphPanel);
		testPanel.add(attributesPanel);
			
		createSplitPanel.add(testPanel);
		
		createSplitPanel.add(buttonsPanel, BorderLayout.SOUTH);
		
		this.add(createSplitPanel);
		
		graphPanel.remove(graphScrollPane);
		
		graphPanel.add(graphScrollPane);
		
		validate();
	}
	
	
	/**
	 * Used to show dialog
	 * 
	 * @param title		title of dialog
	 * @return			true if user want to confirm action
	 */
	public boolean showDialog(String title)
	{
		ok = false;
		
		dialog = new JDialog(new JFrame(), title, true);
		
		int screenWidth = (int)Toolkit.getDefaultToolkit().getScreenSize().getWidth();
		int screenHeight = (int)Toolkit.getDefaultToolkit().getScreenSize().getHeight();
		
		dialog.getContentPane().add(this);
		
		createLabel("Label");
		
		dialog.pack();
		
		dialog.setLocation((int)(screenWidth - dialog.getSize().getWidth()) / 2, (int)(screenHeight - dialog.getSize().getHeight()) / 2);
		
		dialog.setVisible(true);
		return ok;
	}
	
	
	/**
	 * Returns list of properties created as a result of split operation.
	 * 
	 * @return	list of new properties
	 */
	public List<Property> getNewProperties()
	{
		List<Property> newPropertiesList = new ArrayList<Property>();
		
		for (GraphNode graphNode : newPropertiesNodes)
		{
			newPropertiesList.add(graphNode.getProperty());
		}
		
		return newPropertiesList;
	}
	
	
	/**
	 * Returns list of dependencies created as a result of split operation
	 * 
	 * @return	list of new dependencies
	 */
	public List<Dependency> getNewDependenciesList()
	{
		List<Dependency> newDependenciesList = new ArrayList<Dependency>();
		
		for (DefaultEdge edge : newEdgesList)
		{
			Property sourceProperty = null;
			Property destinyProperty = null;
			
			// checks if there are dependencies between ingoing properties and new properties
			for (GraphNode graphNode : ingoingPropertiesNodes)
			{
				if (edge.getSource() == graphNode.getChildAt(0))
				{
					sourceProperty = graphNode.getProperty();
				}
			}
			// checks if there are dependencies between new properties
			for (GraphNode graphNode : newPropertiesNodes)
			{
				if (edge.getSource() == graphNode.getChildAt(0))
				{
					sourceProperty = graphNode.getProperty();
				}
				if (edge.getTarget() == graphNode.getChildAt(0))
				{
					destinyProperty = graphNode.getProperty();
				}
			}
			// checks if there are dependencies between new properties and outgoing properties
			for (GraphNode graphNode : outgoingPropertiesNodes)
			{
				if (edge.getTarget() == graphNode.getChildAt(0))
				{
					destinyProperty = graphNode.getProperty();
				}
			}
			
			if (sourceProperty != null && destinyProperty != null)
			{
				Dependency dependency = new Dependency(sourceProperty, destinyProperty);
				newDependenciesList.add(dependency);
			}
		}
		
		return newDependenciesList;
	}
	
	
	/**
	 * Used to create new graph along with scroll pane
	 */
	private void createSplitGraphScrollPane()
	{
		cellsList = new ArrayList<DefaultGraphCell>();
		
		cellsList.addAll(ingoingPropertiesNodes);
		cellsList.addAll(newPropertiesNodes);
		cellsList.addAll(outgoingPropertiesNodes);
		
		
		DefaultGraphCell[] cells = new DefaultGraphCell[cellsList.size()];
		
		for (int i = 0 ; i < cellsList.size() ; i++)
		{
			cells[i] = cellsList.get(i);
		}
		
		graph = new JGraph(new DefaultGraphModel());
		
		graph.getGraphLayoutCache().insert(cells);
		
	
		// calculates node's bounds
		JGraphFacade facade = new JGraphFacade(graph);
		
		com.jgraph.layout.simple.SimpleGridLayout graphTreeLayout = new com.jgraph.layout.simple.SimpleGridLayout();
		
		graphTreeLayout.run(facade);
		
		Map nested = facade.createNestedMap(true, true);
		
		graph.getGraphLayoutCache().edit(nested);

		
		// get maximal width of ingoing properties, maximal width of outgoing properties,
		// height of ingoing properties section and height of outgoing properties section
		int maxIngoingGraphNodeWidth = 0;
		int maxOutgoingGraphNodeWidth = 0;
		
		int ingoingPropertiesHeight = 0;
		int outgoingPropertiesHeight = 0;
		
		for (GraphNode ingoingGraphNode : ingoingPropertiesNodes)
		{
			Rectangle2D bounds = GraphConstants.getBounds(ingoingGraphNode.getAttributes());
		
			if ((int)bounds.getWidth() > maxIngoingGraphNodeWidth) maxIngoingGraphNodeWidth = (int)bounds.getWidth();
			
			ingoingPropertiesHeight += (int)bounds.getHeight();
			ingoingPropertiesHeight += 100;
		}
		
		if (ingoingPropertiesHeight > 50) ingoingPropertiesHeight -= 100;
		
		
		for (GraphNode outgoingGraphNode : outgoingPropertiesNodes)
		{
			Rectangle2D bounds = GraphConstants.getBounds(outgoingGraphNode.getAttributes());
			
			if ((int)bounds.getWidth() > maxOutgoingGraphNodeWidth) maxOutgoingGraphNodeWidth = (int)bounds.getWidth();
			
			outgoingPropertiesHeight += (int)bounds.getHeight();
			outgoingPropertiesHeight += 100;
		}
		
		if (outgoingPropertiesHeight > 50) outgoingPropertiesHeight -= 100;
	
		int centerY = 120;
		
		if ((ingoingPropertiesHeight + 50) / 2 > centerY) centerY = (ingoingPropertiesHeight + 50) / 2;
		if ((outgoingPropertiesHeight + 50) / 2 > centerY) centerY = (outgoingPropertiesHeight + 50) / 2;
		
		
		// moves ingoing properties to valid positions
		int actualIngoingPropertyHeight = centerY - ingoingPropertiesHeight / 2;
		
		for (GraphNode ingoingGraphNode : ingoingPropertiesNodes)
		{
			Rectangle2D newIngoingPropertyBounds = new Rectangle2D.Double(
					50, 
					actualIngoingPropertyHeight, 
					maxIngoingGraphNodeWidth, 
					GraphConstants.getBounds(ingoingGraphNode.getAttributes()).getWidth());
			
			GraphConstants.setBounds(ingoingGraphNode.getAttributes(), newIngoingPropertyBounds);
			
			GraphConstants.setInset(ingoingGraphNode.getAttributes(), 10);
			GraphConstants.setBorderColor(ingoingGraphNode.getAttributes(), Color.LIGHT_GRAY);
			GraphConstants.setAutoSize(ingoingGraphNode.getAttributes(), true);
			GraphConstants.setEditable(ingoingGraphNode.getAttributes(), false);
			
			actualIngoingPropertyHeight += GraphConstants.getBounds(ingoingGraphNode.getAttributes()).getHeight();
			actualIngoingPropertyHeight += 100;
		}
		
		
		// moves new properties to valid positions
		int actualNewPropertyHeight = centerY - 70;
		
		for (GraphNode newGraphNode : newPropertiesNodes)
		{
			Rectangle2D newPropertyBounds = new Rectangle2D.Double(
					150 + maxIngoingGraphNodeWidth, 
					actualNewPropertyHeight, 
					30, 
					20);
			
			GraphConstants.setBounds(newGraphNode.getAttributes(), newPropertyBounds);
			
			GraphConstants.setInset(newGraphNode.getAttributes(), 10);
			GraphConstants.setBorderColor(newGraphNode.getAttributes(), Color.BLACK);
			GraphConstants.setAutoSize(newGraphNode.getAttributes(), true);
			GraphConstants.setEditable(newGraphNode.getAttributes(), false);
			
			actualNewPropertyHeight += GraphConstants.getBounds(newGraphNode.getAttributes()).getHeight();
			actualNewPropertyHeight += 100;
		}
		
		
		// moves outgoing properties to valid positions
		int actualOutgoingPropertyHeight = centerY - outgoingPropertiesHeight / 2;
		
		for (GraphNode outgoingGraphNode : outgoingPropertiesNodes)
		{
			Rectangle2D newOutgoingPropertyBounds = new Rectangle2D.Double(
					280 + maxOutgoingGraphNodeWidth, 
					actualOutgoingPropertyHeight, 
					maxOutgoingGraphNodeWidth, 
					GraphConstants.getBounds(outgoingGraphNode.getAttributes()).getWidth());
		
			GraphConstants.setBounds(outgoingGraphNode.getAttributes(), newOutgoingPropertyBounds);
			
			GraphConstants.setInset(outgoingGraphNode.getAttributes(), 10);
			GraphConstants.setBorderColor(outgoingGraphNode.getAttributes(), Color.LIGHT_GRAY);
			GraphConstants.setAutoSize(outgoingGraphNode.getAttributes(), true);
			GraphConstants.setEditable(outgoingGraphNode.getAttributes(), false);
			
			//GraphConstants.setSelectable(outgoingGraphNode.getAttributes(), true);
			
			actualOutgoingPropertyHeight += GraphConstants.getBounds(outgoingGraphNode.getAttributes()).getHeight();
			actualOutgoingPropertyHeight += 100;
		}
		
		
		for (DefaultGraphCell graphCell : cellsList)
		{
			GraphConstants.setSelectable(graphCell.getAttributes(), true);
		}
		
		cells = new DefaultGraphCell[cellsList.size()];
		
		for (int i = 0 ; i < cellsList.size() ; i++)
		{
			cells[i] = cellsList.get(i);
		}
		
		graph = new JGraph(new DefaultGraphModel());
		
		graph.getGraphLayoutCache().insert(cells);
		
		graph.validate();
		graph.repaint();
		
		graph.addMouseListener(new SplitMouseListener());
		graph.addKeyListener(new SplitKeyListener());
		
		graphScrollPane = new JScrollPane(graph);
		
		graphScrollPane.setPreferredSize(new Dimension(500,500));
		
		graph.requestFocus();
	}
	
	
	/**
	 * Used to clean created edges
	 */
	private void cleanEdges()
	{
		cellsList.removeAll(newEdgesList);
		
		DefaultGraphCell[] cells = new DefaultGraphCell[cellsList.size()];
		
		for (int i = 0 ; i < cellsList.size() ; i++)
		{
			cells[i] = cellsList.get(i);
		}
		
		graph = new JGraph(new DefaultGraphModel());
		
		graph.getGraphLayoutCache().insert(cells);
		
		graph.validate();
		graph.repaint();
		
		graph.addMouseListener(new SplitMouseListener());
		graph.addKeyListener(new SplitKeyListener());
		
		graphPanel.remove(graphScrollPane);
		
		graphScrollPane = new JScrollPane(graph);
		
		graphScrollPane.setPreferredSize(new Dimension(500, 500));
		
		graphPanel.add(graphScrollPane);
		
		graph.requestFocus();
		
		validate();
		repaintGraph();
		
		newEdgesList = new ArrayList<DefaultEdge>();
		
		repaintGraph();
	}
	
	
	/**
	 * Removes selected nodes and edges
	 */
	private void removeSelected()
	{
		// list of removed nodes
		List<GraphNode> removedNodes = new ArrayList<GraphNode>();
		
		for (GraphNode node : newPropertiesNodes)
		{
			if (graph.getSelectionModel().isCellSelected(node))
			{
				removedNodes.add(node);
			}
		}
		
		
		// list of removed edges
		List<DefaultEdge> removedEdges = new ArrayList<DefaultEdge>();
		
		for (DefaultEdge edge : newEdgesList)
		{
			// selected edges have to be removed
			if (graph.getSelectionModel().isCellSelected(edge))
			{
				removedEdges.add(edge);
			}
			
			// if source node or target node of edge is removed, edge also have to be removed
			for (GraphNode node : removedNodes)
			{
				if ((edge.getSource() == node.getChildAt(0) || edge.getTarget() == node.getChildAt(0)) &&
						!removedEdges.contains(edge))
				{
					removedEdges.add(edge);
				}
			}
		}
		
		
		// check if user removes all new properties 
		if (newPropertiesNodes.size() - removedNodes.size() <= 0)
		{
			JOptionPane.showMessageDialog(dialog, "You can not remove all new properties", "Error", JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		// user have to select at least one node or edge to remove
		if (removedEdges.size() == 0 && removedNodes.size() == 0)
		{
			JOptionPane.showMessageDialog(dialog, "Select node or edge to remove", "Error", JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		
		newEdgesList.removeAll(removedEdges);
		
		
		// create actual list of attributes to use
		for (GraphNode removedNode : removedNodes)
		{
			Property property = removedNode.getProperty();
			
			// get list of selected property attributes
			List<String> attributesNames = property.getAttributesNames();
			
			oldAttributesNames.addAll(attributesNames);	
		}
		
		newPropertiesNodes.removeAll(removedNodes);
		
		
		Collections.sort(oldAttributesNames);
		
		List<String> attributesNames = new ArrayList<String>();
		
		newAttributesList.setListData(attributesNames.toArray());
		
		attributesRightPanel.remove(newAttributesScrollPane);
		
		newAttributesScrollPane = new JScrollPane(newAttributesList);
		newAttributesScrollPane.setPreferredSize(new Dimension(150,200));
		newAttributesScrollPane.setWheelScrollingEnabled(true);
		
		attributesRightPanel.add(newAttributesScrollPane);
	
		oldAttributesList.setListData(oldAttributesNames.toArray());
	
		attributesLeftPanel.remove(oldAttributesScrollPane);
	
		oldAttributesScrollPane = new JScrollPane(oldAttributesList);
		oldAttributesScrollPane.setPreferredSize(new Dimension(150,200));
		oldAttributesScrollPane.setWheelScrollingEnabled(true);
	
		attributesLeftPanel.add(oldAttributesScrollPane);
		
		
		// create new graph
		cellsList.removeAll(removedNodes);
		cellsList.removeAll(removedEdges);
		
		markGraphNode(null);
		
		DefaultGraphCell[] cells = new DefaultGraphCell[cellsList.size()];
		
		for (int i = 0 ; i < cellsList.size() ; i++)
		{
			cells[i] = cellsList.get(i);
		}
		
		
		graph = new JGraph(new DefaultGraphModel());
		
		graph.getGraphLayoutCache().insert(cells);
		
		graph.validate();
		graph.repaint();
		
		graph.addMouseListener(new SplitMouseListener());
		graph.addKeyListener(new SplitKeyListener());
		
		graphPanel.remove(graphScrollPane);
		
		graphScrollPane = new JScrollPane(graph);
		
		graphScrollPane.setPreferredSize(new Dimension(500, 500));
		
		graphPanel.add(graphScrollPane);
		
		graph.requestFocus();
		
		validate();
		
		repaintGraph();
	}
	
	
	/**
	 * Used to create label
	 *  
	 * @param labelInfo	text presented on label
	 */
	private void createLabel(String labelInfo)
	{
			if (labelPanel != null)
			{
				this.getParent().remove(labelPanel);
			}
		
			labelPanel = new JPanel();
		
			JLabel label = null;
		
			boolean isEdgesSelected = false;
      	
			for (DefaultEdge edge : newEdgesList)
			{
				if (graph.getSelectionModel().isCellSelected(edge))
				{
					isEdgesSelected = true;
					break;
				}
			}
      	
			if (isEdgesSelected && labelInfo != "")
			{
				label = new JLabel("Press 'Delete' to remove selected edges");
			}
      		else
      		{
      			label = new JLabel("Label");
      			if (labelInfo == "") label = new JLabel("Label");
    			else label = new JLabel(labelInfo);
      		}
      	
			labelPanel.add(label);
			labelPanel.setBorder(BorderFactory.createEtchedBorder());
			this.getParent().add(labelPanel, BorderLayout.NORTH);
		
			dialog.pack();
			dialog.validate();
			dialog.setResizable(false);
	}
	
	
	/**
	 * Used to mark given graph node
	 * 
	 * @param graphNode	graph node that have to be marked, null to unmark all nodes
	 */
	private void markGraphNode(GraphNode graphNode)
	{	
		Map nested=new Hashtable();
		
		Map inactiveMap = new Hashtable();
	
		GraphConstants.setBackground(inactiveMap, Color.WHITE);
		GraphConstants.setGradientColor(inactiveMap, Color.WHITE);
		GraphConstants.setOpaque(inactiveMap, true);
	
		Map activeMap = new Hashtable();
		
		GraphConstants.setBackground(activeMap, Color.GREEN);
		GraphConstants.setGradientColor(activeMap, Color.WHITE);
		GraphConstants.setOpaque(activeMap, true);
		
		if (graphNode == null)
		{
			for (int i = 0 ; i < cellsList.size() ; i++)
			{
				if (cellsList.get(i) instanceof GraphNode)
				{
					nested.put(cellsList.get(i), inactiveMap);
				}
			}
			
			independentGraphNode = null;
		}
		else
		{
			for (int i = 0 ; i < cellsList.size() ; i++)
			{
				if (cellsList.get(i) instanceof GraphNode)
				{
					if (cellsList.get(i) == graphNode)
					{
						nested.put(cellsList.get(i), activeMap);
					}
					else
					{
						nested.put(cellsList.get(i), inactiveMap);
					}
				}
			}
		}
		
		graph.getGraphLayoutCache().edit(nested,null,null,null);
		graph.repaint();
  	
		validate();
	}
	
	
	/**
	 * Used to add attributes to selected property
	 */
	private void addAttributes()
	{
		if (newPropertyNumber >= 0)
		{
			// get property that is represented by selected node
			GraphNode newPropertyNode = newPropertiesNodes.get(newPropertyNumber);
			
			Property property = newPropertyNode.getProperty();
			
			// get list of selected property attributes
			List<String> attributesNames = property.getAttributesNames();
			
			Object[] selectedItems = oldAttributesList.getSelectedValues();
		
			// add selected new attributes to selected property
			for (int i = 0 ; i < selectedItems.length ; i++)
			{
				String selectedAttribute = (String)selectedItems[i];
			
				attributesNames.add(selectedAttribute.trim());
				
				oldAttributesNames.remove(selectedAttribute.trim());	
			}
			
			Collections.sort(attributesNames);
			
			GraphNode selectedGraphNode = newPropertiesNodes.get(newPropertyNumber);
			
			List<Attribute> newPropertyNodeAttributes = new ArrayList<Attribute>();
			
			for (String newPropertyNodeAttributeName : attributesNames)
			{
				newPropertyNodeAttributes.add(new Attribute(newPropertyNodeAttributeName));
			}
		
			selectedGraphNode.setProperty(new Property(newPropertyNodeAttributes));
			
		
			// refresh attributes lists
			newAttributesList.setListData(attributesNames.toArray());
			
			attributesRightPanel.remove(newAttributesScrollPane);
			
			newAttributesScrollPane = new JScrollPane(newAttributesList);
			newAttributesScrollPane.setPreferredSize(new Dimension(150,200));
			newAttributesScrollPane.setWheelScrollingEnabled(true);
			
			attributesRightPanel.add(newAttributesScrollPane);
		
			oldAttributesList.setListData(oldAttributesNames.toArray());
		
			attributesLeftPanel.remove(oldAttributesScrollPane);
		
			oldAttributesScrollPane = new JScrollPane(oldAttributesList);
			oldAttributesScrollPane.setPreferredSize(new Dimension(150,200));
			oldAttributesScrollPane.setWheelScrollingEnabled(true);
		
			attributesLeftPanel.add(oldAttributesScrollPane);
			
			repaintGraph();
		}
		else
		{
			JOptionPane.showMessageDialog(dialog, "Select property", "Error", JOptionPane.ERROR_MESSAGE);
		}
	}
	
	
	/**
	 * Used to remove selected attributes from selected property
	 */
	private void removeAttributes()
	{
		if (newPropertyNumber >= 0)
		{
			// get property that is represented by selected node
			GraphNode newPropertyNode = newPropertiesNodes.get(newPropertyNumber);
			
			Property property = newPropertyNode.getProperty();
			
			// get list of selected property attributes
			List<String> attributesNames = property.getAttributesNames();
			
			// get selected attributes and remove them from property attributes list
			Object[] selectedItems = newAttributesList.getSelectedValues();
		
			for (int i = 0 ; i < selectedItems.length ; i++)
			{
				String selectedAttribute = (String)selectedItems[i];
			
				attributesNames.remove(selectedAttribute.trim());
				
				oldAttributesNames.add(selectedAttribute.trim());	
			}
			
			Collections.sort(attributesNames);
			Collections.sort(oldAttributesNames);
			
			GraphNode selectedGraphNode = newPropertiesNodes.get(newPropertyNumber);
			
			List<Attribute> newPropertyNodeAttributes = new ArrayList<Attribute>();
			
			for (String newPropertyNodeAttributeName : attributesNames)
			{
				newPropertyNodeAttributes.add(new Attribute(newPropertyNodeAttributeName));
			}
		
			selectedGraphNode.setProperty(new Property(newPropertyNodeAttributes));
		
			newAttributesList.setListData(attributesNames.toArray());
			
			attributesRightPanel.remove(newAttributesScrollPane);
			
			newAttributesScrollPane = new JScrollPane(newAttributesList);
			newAttributesScrollPane.setPreferredSize(new Dimension(150,200));
			newAttributesScrollPane.setWheelScrollingEnabled(true);
			
			attributesRightPanel.add(newAttributesScrollPane);
		
			oldAttributesList.setListData(oldAttributesNames.toArray());
		
			attributesLeftPanel.remove(oldAttributesScrollPane);
		
			oldAttributesScrollPane = new JScrollPane(oldAttributesList);
			oldAttributesScrollPane.setPreferredSize(new Dimension(150,200));
			oldAttributesScrollPane.setWheelScrollingEnabled(true);
		
			attributesLeftPanel.add(oldAttributesScrollPane);
			
			repaintGraph();
		}
		else
		{
			JOptionPane.showMessageDialog(dialog, "Select property", "Error", JOptionPane.ERROR_MESSAGE);
		}
	}
	
	
	/**
	 * Used to repaint graph
	 */
	private void repaintGraph()
	{
		markGraphNode(null);
		
		graph.repaint();
		
		validate();
	}
	
	
	/**
	 * Used to validate split operation
	 * 
	 * @return	true if split operation is valid, false otherwise
	 */
	private boolean validateSplit()
	{
		// all attributes have to be assign to new property
		if (oldAttributesNames.size() > 0)
		{
			JOptionPane.showMessageDialog(dialog, "Assign all attributes", "Error", JOptionPane.ERROR_MESSAGE);
			return false;
		}
		
		// all new properties have to have at least one attribute
		for (GraphNode graphNode : newPropertiesNodes)
		{
			if (graphNode.getProperty().getAttributes().size() == 0)
			{
				JOptionPane.showMessageDialog(dialog, "Some of new properties are empty", "Error", JOptionPane.ERROR_MESSAGE);
				return false;
			}
		}
		
		// as a result of split operation at least two new properties have to be created
		if (newPropertiesNodes.size() < 2)
		{
			JOptionPane.showMessageDialog(dialog, "You must create at least 2 new properties", "Error", JOptionPane.ERROR_MESSAGE);
			
			return false;
		}
		
		
		// if there are some ingoing properties, each of new properties have to be connected with at least one ingoing property
		if (ingoingPropertiesNodes.size() > 0)
		{
			List<GraphNode> connectedNodes = new ArrayList<GraphNode>(); 
			
			for (GraphNode graphNode : ingoingPropertiesNodes)
			{
				recurentConnectNode(graphNode, connectedNodes, true);
			}
			
			if (!connectedNodes.containsAll(newPropertiesNodes))
			{
				JOptionPane.showMessageDialog(dialog, "Created dependencies are not valid", "Error", JOptionPane.ERROR_MESSAGE);
				
				return false;
			}
		}
		
		// if there are some outgoing properties, each of new properties have to be connected with at least one outgoing property
		if (outgoingPropertiesNodes.size() > 0)
		{
			List<GraphNode> connectedNodes = new ArrayList<GraphNode>(); 
			
			for (GraphNode graphNode : outgoingPropertiesNodes)
			{
				recurentConnectNode(graphNode, connectedNodes, false);
			}
			
			if (!connectedNodes.containsAll(newPropertiesNodes))
			{
				JOptionPane.showMessageDialog(dialog, "Created dependencies are not valid", "Error", JOptionPane.ERROR_MESSAGE);
				
				return false;
			}
		}
		
		return true;
	}
	
	
	/**
	 * Creates list of nodes that are connected to given node
	 * 
	 * @param connectedNode			node
	 * @param connectedNodesList	list of connected nodes
	 * @param forward				true to forward-connect, false otherwise
	 */
	private void recurentConnectNode(GraphNode connectedNode, List<GraphNode> connectedNodesList, boolean forward)
	{
		connectedNodesList.add(connectedNode);
		
		// list of all nodes
		List<GraphNode> allNodes = new ArrayList<GraphNode>();
		
		allNodes.addAll(ingoingPropertiesNodes);
		allNodes.addAll(newPropertiesNodes);
		allNodes.addAll(outgoingPropertiesNodes);
		
		// if forward-connect, check all edges that have given node as a source node
		if (forward)
		{
			for (DefaultEdge edge : newEdgesList)
			{
				for (GraphNode dependentGraphNode : allNodes)
				{
					if (connectedNode.getChildAt(0) == edge.getSource() &&
							dependentGraphNode.getChildAt(0) == edge.getTarget() &&
							!connectedNodesList.contains(dependentGraphNode))
					{
						recurentConnectNode(dependentGraphNode, connectedNodesList, forward);
					}
				}
			}
		}
		// if backward-connect, check all edges that have given node as a target node
		else
		{
			for (DefaultEdge edge : newEdgesList)
			{
				for (GraphNode independentGraphNode : allNodes)
				{
					if (connectedNode.getChildAt(0) == edge.getTarget() &&
							independentGraphNode.getChildAt(0) == edge.getSource() &&
							!connectedNodesList.contains(independentGraphNode))
					{
						recurentConnectNode(independentGraphNode, connectedNodesList, forward);
					}
				}
			}
		}
	}
	
	
	/**
	 * Listener of key events
	 * 
	 * @author Szymon Książek
	 *
	 */
	private class SplitKeyListener extends KeyAdapter
	{
		public void keyPressed(KeyEvent e)
		{
			if (e.getKeyCode() == KeyEvent.VK_DELETE)
			{
				removeSelected();
			}
		}
	}
	
	
	/**
	 * Listener of mouse events
	 * 
	 * @author Szymon Książek
	 *
	 */
	private class SplitMouseListener extends MouseAdapter 
	{
		public void mousePressed(MouseEvent e)
		{
			if (e.getButton() == MouseEvent.BUTTON1)
			{
				Object cell = graph.getFirstCellForLocation(e.getX(), e.getY());
				
				// if some node have been clicked
				if (cell != null && cell instanceof GraphNode)
				{
					GraphNode selectedGraphNode = (GraphNode)cell;
					
					if (newPropertiesNodes.contains(selectedGraphNode) && e.getClickCount() == 2 && !e.isControlDown())
					{
						markGraphNode(null);
						
						// refresh attributes lists
						newPropertyNumber = newPropertiesNodes.indexOf(selectedGraphNode);
						
						Property property = selectedGraphNode.getProperty();
					
						List<String> attributesNames = property.getAttributesNames();
					
						newAttributesList.setListData(attributesNames.toArray());
					
						attributesRightPanel.remove(newAttributesScrollPane);
					
						newAttributesScrollPane = new JScrollPane(newAttributesList);
						newAttributesScrollPane.setPreferredSize(new Dimension(150,200));
						newAttributesScrollPane.setWheelScrollingEnabled(true);
					
						attributesRightPanel.add(newAttributesScrollPane);
				
						Map nested=new Hashtable();
					
						Map inactiveMap = new Hashtable();

						GraphConstants.setInset(inactiveMap, 10);
						GraphConstants.setBorderColor(inactiveMap, Color.BLACK);
						GraphConstants.setAutoSize(inactiveMap, true);
					
						Map activeMap = new Hashtable();

						GraphConstants.setInset(activeMap, 10);
						GraphConstants.setBorderColor(activeMap, Color.RED);
						GraphConstants.setAutoSize(activeMap, true);
					 
						for (GraphNode graphNode : newPropertiesNodes)
						{
							if (graphNode != selectedGraphNode)
							{
								graphNode.setSelected(false);
								nested.put(graphNode, inactiveMap);
							}
							else
							{
								// if selected node have been selected, unselect it
								if (selectedGraphNode.isSelected())
								{
									attributesRightPanel.remove(newAttributesScrollPane);
								
									newAttributesScrollPane = new JScrollPane(new JList());
									newAttributesScrollPane.setPreferredSize(new Dimension(150,200));
									newAttributesScrollPane.setWheelScrollingEnabled(true);
								
									attributesRightPanel.add(newAttributesScrollPane);
								
									validate();
								
									graphNode.setSelected(false);
								
									nested.put(graphNode, inactiveMap);
								}
								// otherwise select graph node
								else
								{
									graphNode.setSelected(true);
								
									nested.put(graphNode, activeMap);
								}
							}
						}
					
						graph.getGraphLayoutCache().edit(nested,null,null,null);
						
						graph.requestFocus();
						
						validate();
					}
					
					if (e.isControlDown())
					{
						Map nested=new Hashtable();
						
						Map inactiveMap = new Hashtable();

						GraphConstants.setInset(inactiveMap, 10);
						GraphConstants.setBorderColor(inactiveMap, Color.BLACK);
						GraphConstants.setAutoSize(inactiveMap, true);
 
						for (GraphNode graphNode : newPropertiesNodes)
						{
							graphNode.setSelected(false);
							
							nested.put(graphNode, inactiveMap);
								
								attributesRightPanel.remove(newAttributesScrollPane);
								
								newAttributesScrollPane = new JScrollPane(new JList());
								newAttributesScrollPane.setPreferredSize(new Dimension(150,200));
								newAttributesScrollPane.setWheelScrollingEnabled(true);
								
								attributesRightPanel.add(newAttributesScrollPane);
								
								validate();
						}
					
						graph.getGraphLayoutCache().edit(nested,null,null,null);
						
						
						// if there is no selected node, select node and change label
						if (independentGraphNode == null)
						{
							independentGraphNode = selectedGraphNode;
							
							markGraphNode(selectedGraphNode);
							
							graph.getSelectionModel().clearSelection();
							
							createLabel("'Ctrl' + select second property");
						}
						// if there is selected graph node, create new edge
						else
						{
							if (ingoingPropertiesNodes.contains(independentGraphNode))
							{
								if (newPropertiesNodes.contains(selectedGraphNode))
								{
									addNewEdge(independentGraphNode, selectedGraphNode);
								}
							}
							else if (newPropertiesNodes.contains(independentGraphNode))
							{
								if (ingoingPropertiesNodes.contains(selectedGraphNode))
								{
									addNewEdge(selectedGraphNode, independentGraphNode);
								}
								else if (outgoingPropertiesNodes.contains(selectedGraphNode))
								{
									addNewEdge(independentGraphNode, selectedGraphNode);
								}
								else if (newPropertiesNodes.contains(selectedGraphNode))
								{
									addNewEdge(independentGraphNode, selectedGraphNode);
								}
							}
							else if (outgoingPropertiesNodes.contains(independentGraphNode))
							{
								if (newPropertiesNodes.contains(selectedGraphNode))
								{
									addNewEdge(selectedGraphNode, independentGraphNode);
								}
							}
						}
					}
				}
				// if none of nodes have been clicked
				else
				{
					newPropertyNumber = -1;
					
					markGraphNode(null);
					
					Map nested=new Hashtable();
					
					Map inactiveMap = new Hashtable();

					GraphConstants.setInset(inactiveMap, 10);
					GraphConstants.setBorderColor(inactiveMap, Color.BLACK);
					GraphConstants.setAutoSize(inactiveMap, true);

					for (GraphNode graphNode : newPropertiesNodes)
					{
						graphNode.setSelected(false);
						nested.put(graphNode, inactiveMap);
					}
					
					attributesRightPanel.remove(newAttributesScrollPane);
					
					newAttributesScrollPane = new JScrollPane(new JList());
					newAttributesScrollPane.setPreferredSize(new Dimension(150,200));
					newAttributesScrollPane.setWheelScrollingEnabled(true);
				
					attributesRightPanel.add(newAttributesScrollPane);
				
					validate();
				
					graph.getGraphLayoutCache().edit(nested,null,null,null);
					
					createLabel("'Ctrl' + click to add dependency");
				}
				
				graph.requestFocus();
			}
			
			// if right button have been clicked on empty space, show popup menu
			if (e.getButton() == MouseEvent.BUTTON3)
			{
				Object cell = graph.getFirstCellForLocation(e.getX(), e.getY());
				
				if (cell == null)
				{
					JPopupMenu popupMenu = new JPopupMenu();
					
					JMenuItem addRootNodeItem = new JMenuItem("Add property");
					addRootNodeItem.setMnemonic('A');
					addRootNodeItem.addActionListener(new AddPropertyAction(e.getPoint()));
					addRootNodeItem.setIcon(arranger.getIcon("add_property"));
					popupMenu.add(addRootNodeItem);
					
					popupMenu.show(e.getComponent(), e.getX(), e.getY());
				}
			}
		}
		
		public void mouseEntered(MouseEvent e)
		{
			createLabel("'Ctrl' + click to add dependency");
			
			graph.requestFocus();			
		}
		
		public void mouseExited(MouseEvent e)
		{
			independentGraphNode = null;
			createLabel("");
			markGraphNode(null);
			graph.requestFocus(false);
		}
	}
	
	
	/**
	 * Action class used to add new property
	 * 
	 * @author Szymon Książek
	 */
	private class AddPropertyAction  extends AbstractAction
	{
		private static final long serialVersionUID = 1L;
		
		private Point point = null;
		
		
		public AddPropertyAction(Point point)
		{
			this.point = point;
		}
		
		public void actionPerformed(ActionEvent event)
		{
			GraphNode newGraphNode = new GraphNode(new Property(new ArrayList<Attribute>()));
			
			newPropertiesNodes.add(newGraphNode);
			
			cellsList.add(newGraphNode);
			
			DefaultGraphCell[] cells = new DefaultGraphCell[cellsList.size()];
			
			for (int i = 0 ; i < cellsList.size() ; i++)
			{
				cells[i] = cellsList.get(i);
			}
			
			Rectangle2D newPropertyBounds = new Rectangle2D.Double(
					(int)point.getX() - 15, 
					(int)point.getY() - 10, 
					30, 
					20);
			
			GraphConstants.setBounds(newGraphNode.getAttributes(), newPropertyBounds);
			
			GraphConstants.setInset(newGraphNode.getAttributes(), 10);
			GraphConstants.setBorderColor(newGraphNode.getAttributes(), Color.BLACK);
			GraphConstants.setAutoSize(newGraphNode.getAttributes(), true);
			GraphConstants.setEditable(newGraphNode.getAttributes(), false);
			
			graph = new JGraph(new DefaultGraphModel());
			
			graph.getGraphLayoutCache().insert(cells);
			
			graph.validate();
			graph.repaint();
			
			graph.addMouseListener(new SplitMouseListener());
			graph.addKeyListener(new SplitKeyListener());
			
			graphPanel.remove(graphScrollPane);
			
			graphScrollPane = new JScrollPane(graph);
			
			graphScrollPane.setPreferredSize(new Dimension(500, 500));
			
			graphPanel.add(graphScrollPane);
			
			graph.requestFocus();
			
			validate();
		}
	}
	
	
	/**
	 * Adds new edge between given nodes
	 * 
	 * @param independentGraphNode
	 * @param selectedGraphNode
	 */
	private void addNewEdge(GraphNode independentGraphNode, GraphNode selectedGraphNode)
	{
		boolean edgeExist = false;
		
		// checks if edge between given nodes already exists
		for (DefaultEdge edge : newEdgesList)
		{
			if (edge.getSource() == independentGraphNode.getChildAt(0) &&
					edge.getTarget() == selectedGraphNode.getChildAt(0))
			{
				edgeExist = true;
				break;
			}
			
			if (edge.getSource() == selectedGraphNode.getChildAt(0) &&
					edge.getTarget() == independentGraphNode.getChildAt(0))
			{
				edgeExist = true;
				break;
			}
		}
		
		// if no, add new edge
		if (!edgeExist)
		{
			DefaultEdge edge = new DefaultEdge();
			edge.setSource(independentGraphNode.getChildAt(0));
			edge.setTarget(selectedGraphNode.getChildAt(0));
			GraphConstants.setLineEnd(edge.getAttributes(), GraphConstants.ARROW_CLASSIC);
			GraphConstants.setEndFill(edge.getAttributes(), true);
			GraphConstants.setLineColor(edge.getAttributes(), Color.BLACK);
			
			newEdgesList.add(edge);
			
			markGraphNode(null);
			
			cellsList.removeAll(newEdgesList);
			cellsList.addAll(newEdgesList);
			
			DefaultGraphCell[] cells = new DefaultGraphCell[cellsList.size()];
			
			for (int i = 0 ; i < cellsList.size() ; i++)
			{
				cells[i] = cellsList.get(i);
			}
			
			graph = new JGraph(new DefaultGraphModel());
			
			graph.getGraphLayoutCache().insert(cells);
			
			graph.validate();
			graph.repaint();
			
			graph.addMouseListener(new SplitMouseListener());
			graph.addKeyListener(new SplitKeyListener());
			
			graphPanel.remove(graphScrollPane);
			
			graphScrollPane = new JScrollPane(graph);
			
			graphScrollPane.setPreferredSize(new Dimension(500, 500));
			
			graphPanel.add(graphScrollPane);
			
			graph.requestFocus();
			
			validate();
		}
	}
}