/*
 *     Implementation by Szymon Ksiazek <szymonh20@poczta.fm>
 *
 *     Copyright (C) 2006-9 by the HeKatE Project
 *
 *     HJEd has been develped within the HeKatE Project, 
 *     see http://hekate.ia.agh.edu.pl
 *
 *     This file is part of HJEd.
 *
 *     HJEd is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     HJEd is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with HJEd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package pl.edu.agh.hekate.ard.dialogs;

import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;

import pl.edu.agh.hekate.ard.ARDModel;
import pl.edu.agh.hekate.ard.ARDView;
import pl.edu.agh.hekate.ard.Attribute;
import pl.edu.agh.hekate.ard.MainFrame;

/**
 * Dialog used to refactor attributes
 * 
 * @author Szymon Książek
 *
 */
public class RefactorAttributesDialog extends JPanel
{
	private static final long serialVersionUID = 1L;
	
	/**
	 * model reference
	 */
	private ARDModel model;
	
	/**
	 * view reference
	 */
	private ARDView view;
	
	
	/**
	 * true if user want to close dialog and confirm operation
	 */
	private boolean ok;
	
	/**
	 * dialog reference
	 */
	private JDialog dialog;
	
	/**
	 * list that contains names of attributes
	 */
	private List<String> attributesNamesList;
	
	/**
	 * used to present attributes names to user
	 */
	private JList attributesNames;
	
	/**
	 * panel that contains attributes names list
	 */
	private JPanel attributesPanel;
	
	
	/**
	 * Constructor
	 * 
	 * @param model	model reference
	 * @param view	view reference
	 */
	public RefactorAttributesDialog(ARDModel model, ARDView view)
	{
		this.model = model;
		this.view = view;
		
		setLayout(new BorderLayout());
		
		attributesNamesList = new ArrayList<String>();
	
		// create list of all attributes
		List<Attribute> attributesList = model.getAttributesList();
		
		for (Attribute attribute : attributesList)
		{
			attributesNamesList.add(attribute.getName());
		}
		
		// sort attributes by name
		Collections.sort(attributesNamesList);
		
		attributesNames = new JList(attributesNamesList.toArray());
		attributesNames.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		
		attributesPanel = new JPanel();
		attributesPanel.setBorder(new TitledBorder("Attributes names:"));
		
		JScrollPane attributesNamesPanel = new JScrollPane(attributesNames);
		attributesNamesPanel.setPreferredSize(new Dimension(150, 100));
		
		attributesPanel.add(attributesNamesPanel);
		
		add(attributesPanel);
		
		
		JPanel buttonsPanel = new JPanel();
		
		// button used to change selected attribute name
		JButton changeAttributeNameButton = new JButton("Change attribute name");
		
		changeAttributeNameButton.addActionListener(new
				ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						changeAttributeName();
					}
				});
		
		
		// button used to remove selected attribute
		JButton removeAttributeButton = new JButton("Remove attribute");
		
		removeAttributeButton.addActionListener(new
				ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						removeAttribute();
					}
				});
		
		
		// button used to close dialog
		JButton OKButton = new JButton("OK");
		
		OKButton.addActionListener(new
				ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						ok = true;
						dialog.setVisible(false);
					}
				});
		
		buttonsPanel.add(changeAttributeNameButton);
		buttonsPanel.add(removeAttributeButton);
		buttonsPanel.add(OKButton);
		
		add(buttonsPanel, BorderLayout.SOUTH);
		
		this.setBorder(new EmptyBorder(20, 30, 20, 30));
	}
	
	
	/**
	 * Method used to change selected attribute name
	 */
	private void changeAttributeName()
	{
		// get selected attribute name
		String oldAttributeName = (String)attributesNames.getSelectedValue();
		
		if (oldAttributeName != null)
		{
			this.setCursor(new Cursor(Cursor.WAIT_CURSOR));
			
			String newAttributeName = "";
			
			boolean correctNewAttributeName = true;
		
			// get new name of attribute
			newAttributeName = JOptionPane.showInputDialog("Enter new name for attribute '" + oldAttributeName + "'");
			
			if (newAttributeName != null)
			{
				String errorMessage = "";
			
				// attribute name can not be empty string
				if (newAttributeName.equals(""))
				{
					correctNewAttributeName = false;
					errorMessage = "New name can no be empty. Enter new name for attribute '" + oldAttributeName + "'";
				}
			
				if (correctNewAttributeName)
				{
					Attribute attribute = model.getAttribute(newAttributeName.trim());
				
					// new attribute name can not exists as attribute name for some other attribute
					if (attribute != null)
					{
						correctNewAttributeName = false;
						errorMessage = "Attribute exists. Enter new name for attribute '" + oldAttributeName + "'";
					}
				}
			
				// show dialog while user enteres valid new attribute name
				while(!correctNewAttributeName)
				{
					newAttributeName = JOptionPane.showInputDialog(errorMessage);
				
					correctNewAttributeName = true;
					
					if (newAttributeName != null)
					{
					
						if (newAttributeName.equals(""))
						{
							correctNewAttributeName = false;
							errorMessage = "New name can no be empty. Enter new name for attribute '" + oldAttributeName + "'";
						}
				
						if (correctNewAttributeName)
						{
							Attribute attribute = model.getAttribute(newAttributeName.trim());
					
							if (attribute != null)
							{
								correctNewAttributeName = false;
								errorMessage = "Attribute exists. Enter new name for attribute '" + oldAttributeName + "'";
							}	
						}
					}
					else
					{
						correctNewAttributeName = false;
						break;
					}
				}	
			}
			else
			{
				correctNewAttributeName = false;
			}
			
			if (correctNewAttributeName)
			{
				// change attribute name
				model.changeAttributeName(oldAttributeName, newAttributeName);
				
				view.refreshGraphs();
				view.refreshARDGraph();
				
				view.getPersistenceManager().setStateSaved(false);
				
				// update attributes list
				remove(attributesPanel);
				
				attributesNamesList = new ArrayList<String>();
				
				List<Attribute> attributesList = model.getAttributesList();
				
				for (Attribute attribute : attributesList)
				{
					attributesNamesList.add(attribute.getName());
				}
				
				Collections.sort(attributesNamesList);
				
				attributesNames = new JList(attributesNamesList.toArray());
				attributesNames.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
				
				attributesPanel = new JPanel();
				
				attributesPanel.setBorder(new TitledBorder("Attributes names:"));
				
				JScrollPane attributesNamesPanel = new JScrollPane(attributesNames);
				
				attributesNamesPanel.setPreferredSize(new Dimension(150, 100));
				
				attributesPanel.add(attributesNamesPanel);
				
				add(attributesPanel);
				
				validate();
			}
			
			this.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		}
		else
		{
			JOptionPane.showMessageDialog(dialog, "Select attribute", "Error", JOptionPane.ERROR_MESSAGE);
		}
	}
	
	/**
	 * Method used to remove selected attribute
	 */
	private void removeAttribute()
	{
		String removedAttributeName = (String)attributesNames.getSelectedValue();

		if (removedAttributeName != null)
		{
			this.setCursor(new Cursor(Cursor.WAIT_CURSOR));
			
			Attribute removedAttribute = null;

			List<Attribute> attributesList = model.getAttributesList();
			
			for (Attribute attribute : attributesList)
			{
				if (attribute.getName().equals(removedAttributeName))
				{
					removedAttribute = attribute;
					break;
				}
			}
			
			
			model.removeAttribute(removedAttribute);
			
			view.clearGraphs();
			
			view.refreshGraphs();

			// refresh attributes list
			remove(attributesPanel);
			
			attributesNamesList = new ArrayList<String>();
			
			List<Attribute> attrList = model.getAttributesList();
			
			for (Attribute attribute : attrList)
			{
				attributesNamesList.add(attribute.getName());
			}
			
			Collections.sort(attributesNamesList);
			
			attributesNames = new JList(attributesNamesList.toArray());
			
			attributesNames.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			
			attributesPanel = new JPanel();
			attributesPanel.setBorder(new TitledBorder("Attributes names:"));
			
			JScrollPane attributesNamesPanel = new JScrollPane(attributesNames);
			
			attributesNamesPanel.setPreferredSize(new Dimension(150, 100));
			
			attributesPanel.add(attributesNamesPanel);
			
			add(attributesPanel);
			
			validate();
			
			view.getPersistenceManager().setStateSaved(false);
			
			this.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		}
		else
		{
			JOptionPane.showMessageDialog(dialog, "Select attribute", "Error", JOptionPane.ERROR_MESSAGE);
		}
	}
	
	
	public boolean showDialog(String title)
	{
		ok = false;
		
		dialog = new JDialog(new JFrame(), title, true);
		
		int screenWidth = (int)Toolkit.getDefaultToolkit().getScreenSize().getWidth();
		int screenHeight = (int)Toolkit.getDefaultToolkit().getScreenSize().getHeight();
		
		dialog.getContentPane().add(this);
		
		dialog.pack();
		
		dialog.setLocation((int)(screenWidth - dialog.getSize().getWidth()) / 2, (int)(screenHeight - dialog.getSize().getHeight()) / 2);
		
		dialog.setResizable(false);
		
		dialog.setVisible(true);
		
		return ok;
	}
}
