/*
 *     Implementation by Szymon Ksiazek <szymonh20@poczta.fm>
 *
 *     Copyright (C) 2006-9 by the HeKatE Project
 *
 *     HJEd has been develped within the HeKatE Project, 
 *     see http://hekate.ia.agh.edu.pl
 *
 *     This file is part of HJEd.
 *
 *     HJEd is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     HJEd is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with HJEd.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package pl.edu.agh.hekate.ard.dialogs;

import javax.swing.*;

import org.jgraph.JGraph;
import org.jgraph.graph.DefaultEdge;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.DefaultGraphModel;
import org.jgraph.graph.GraphConstants;

import com.jgraph.layout.JGraphFacade;
import com.jgraph.layout.tree.JGraphTreeLayout;

import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import pl.edu.agh.hekate.ard.*;

/**
 * Dialog used to define colors of graphs
 * 
 * @author Szymon Książek
 *
 */
public class SetColorsDialog extends JPanel
{
	private static final long serialVersionUID = 1L;
	
	private Arranger arranger;
	
	// specifies colors of ARD graph
	private Color tempARDMarkedLineColor;
	private Color tempARDUnmarkedLineColor;
	private Color tempARDMarkedBackgroundColor;
	private Color tempARDUnmarkedBackgroundColor;
	private Color tempARDBackgroundColor;
	
	// specifies colors of TPH graph
	private Color tempTPHMarkedLineColor;
	private Color tempTPHUnmarkedLineColor;
	private Color tempTPHMarkedBackgroundColor;
	private Color tempTPHUnmarkedBackgroundColor;
	private Color tempTPHBackgroundColor;
	
	// specifies if gradient is used for background color of ARD/TPH graph nodes
	private boolean tempIsGradientARDCell;
	private boolean tempIsGradientTPHCell;
	
	// elements used on graphs
	private GraphNode firstARDMarkedNode;
	private GraphNode secondARDMarkedNode;
	private GraphNode unmarkedARDNode;
	private DefaultEdge markedARDEdge;
	private DefaultEdge unmarkedARDEdge;
	
	private GraphNode firstTPHMarkedNode;
	private GraphNode secondTPHMarkedNode;
	private GraphNode unmarkedTPHNode;
	private DefaultEdge markedTPHEdge;
	private DefaultEdge unmarkedTPHEdge;
	
	private JGraph ARDGraph;
	private JGraph TPHGraph;
	
	private JCheckBox ARDGradientCheckBox;
	private JCheckBox TPHGradientCheckBox;
	
	private boolean ok;
	private JDialog dialog;
	
	
	public SetColorsDialog(Arranger arranger)
	{
		this.arranger = arranger;
		
		this.tempARDMarkedLineColor = arranger.getARDMarkedLineColor();
		this.tempARDUnmarkedLineColor = arranger.getARDUnmarkedLineColor();
		this.tempARDMarkedBackgroundColor = arranger.getARDMarkedBackgroundColor();
		this.tempARDUnmarkedBackgroundColor = arranger.getARDUnmarkedBackgroundColor();
		this.tempARDBackgroundColor = arranger.getARDBackgroundColor();
		
		this.tempTPHMarkedLineColor = arranger.getTPHMarkedLineColor();
		this.tempTPHUnmarkedLineColor = arranger.getTPHUnmarkedLineColor();
		this.tempTPHMarkedBackgroundColor = arranger.getTPHMarkedBackgroundColor();
		this.tempTPHUnmarkedBackgroundColor = arranger.getTPHUnmarkedBackgroundColor();
		this.tempTPHBackgroundColor = arranger.getTPHBackgroundColor();
		
		this.tempIsGradientARDCell = arranger.isGradientARDCell();
		this.tempIsGradientTPHCell = arranger.isGradientTPHCell();
		
		this.setLayout(new BorderLayout());
		
		JPanel mainPanel = new JPanel();
		
		mainPanel.setLayout(new BorderLayout());
		
		
		//ARD panel	
		JPanel ARDColorsPanel = new JPanel();
		
		ARDColorsPanel.setLayout(new BorderLayout());
		
		ARDColorsPanel.setPreferredSize(new Dimension(400, 450));
		
		List<DefaultGraphCell> ARDCellsList = new ArrayList<DefaultGraphCell>();
		
		List<Attribute> firstMarkedPropertyAttributes = new ArrayList<Attribute>();
		firstMarkedPropertyAttributes.add(new Attribute("Marked property 1"));
		
		List<Attribute> secondMarkedPropertyAttributes = new ArrayList<Attribute>();
		secondMarkedPropertyAttributes.add(new Attribute("Marked property 2"));
		
		List<Attribute> unmarkedPropertyAttributes = new ArrayList<Attribute>();
		unmarkedPropertyAttributes.add(new Attribute("Unmarked property"));
		
		firstARDMarkedNode = new GraphNode(new Property(firstMarkedPropertyAttributes));
		secondARDMarkedNode = new GraphNode(new Property(secondMarkedPropertyAttributes));
		unmarkedARDNode = new GraphNode(new Property(unmarkedPropertyAttributes));
		
		GraphConstants.setInset(firstARDMarkedNode.getAttributes(), 10);
		GraphConstants.setAutoSize(firstARDMarkedNode.getAttributes(), true);
		GraphConstants.setInset(secondARDMarkedNode.getAttributes(), 10);
		GraphConstants.setAutoSize(secondARDMarkedNode.getAttributes(), true);
		GraphConstants.setInset(unmarkedARDNode.getAttributes(), 10);
		GraphConstants.setAutoSize(unmarkedARDNode.getAttributes(), true);
		
		ARDCellsList.add(firstARDMarkedNode);
		ARDCellsList.add(secondARDMarkedNode);
		ARDCellsList.add(unmarkedARDNode);
		
		markedARDEdge = new DefaultEdge();
		
		markedARDEdge.setSource(firstARDMarkedNode.getChildAt(0));
		markedARDEdge.setTarget(secondARDMarkedNode.getChildAt(0));
		
		ARDCellsList.add(markedARDEdge);
		
		unmarkedARDEdge = new DefaultEdge();
		
		unmarkedARDEdge.setSource(firstARDMarkedNode.getChildAt(0));
		unmarkedARDEdge.setTarget(unmarkedARDNode.getChildAt(0));
		
		ARDCellsList.add(unmarkedARDEdge);
		
		DefaultGraphCell[] ARDCells = new DefaultGraphCell[ARDCellsList.size()];
		
		for (int i = 0 ; i < ARDCellsList.size() ; i++)
		{
			ARDCells[i] = ARDCellsList.get(i);
		}
		
		ARDGraph = new JGraph(new DefaultGraphModel());
		
		ARDGraph.getGraphLayoutCache().insert(ARDCells);
		
		Object ARDRoots[] = new Object[1];
		
		ARDRoots[0] = ARDCells[0];
		
		JGraphFacade facade = new JGraphFacade(ARDGraph, ARDRoots);
		
		JGraphTreeLayout graphTreeLayout = new JGraphTreeLayout();
		graphTreeLayout.setAlignment(SwingConstants.CENTER);
		graphTreeLayout.setOrientation(SwingConstants.WEST);
		graphTreeLayout.setLevelDistance(150);
		graphTreeLayout.setNodeDistance(100);
		
		graphTreeLayout.run(facade);
		
		Map nested = facade.createNestedMap(true, true);
		
		ARDGraph.getGraphLayoutCache().edit(nested);
		
		for (int i = 0 ; i < ARDCells.length ; i++)
		{
			GraphConstants.setSelectable(ARDCells[i].getAttributes(), false);
		}

		cds.jlow.client.graph.GraphJ.move(ARDGraph, ARDCells, 50, 100);
		
		ARDColorsPanel.add(ARDGraph, BorderLayout.CENTER);
		
		
		// create ARD buttons panel
		JPanel ARDButtonsPanel = new JPanel();
		
		ARDButtonsPanel.setLayout(new BorderLayout());
		
		JPanel ARDLineColorsPanel = new JPanel();
		
		// button used to change marked line color on ARD graph
		JButton ARDMarkedLineColorButton = new JButton("Marked line color");
		
		ARDMarkedLineColorButton.addActionListener(new
				ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						Color newTempARDMarkedLineColor = JColorChooser.showDialog(dialog, "Marked ARD line color chooser", tempARDMarkedLineColor);
						
				        if (newTempARDMarkedLineColor != null) 
				        {
				        	tempARDMarkedLineColor = newTempARDMarkedLineColor;
				        	
				        	repaintGraphs();
				        }
					}
				});
		
		ARDLineColorsPanel.add(ARDMarkedLineColorButton);
		
		// button used to change unmarked line color on ARD graph
		JButton ARDUnmarkedLineColorButton = new JButton("Unmarked line color");
		
		ARDUnmarkedLineColorButton.addActionListener(new
				ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						Color newTempARDUnmarkedLineColor = JColorChooser.showDialog(dialog, "Unmarked ARD line color chooser", tempARDUnmarkedLineColor);
						
				        if (newTempARDUnmarkedLineColor != null) 
				        {
				        	tempARDUnmarkedLineColor = newTempARDUnmarkedLineColor;
				        	
				        	repaintGraphs();
				        }
					}
				});
		
		ARDLineColorsPanel.add(ARDUnmarkedLineColorButton);
		
		ARDButtonsPanel.add(ARDLineColorsPanel, BorderLayout.NORTH);


		JPanel ARDPropertyBackgroundColorsPanel = new JPanel();
		
		// button used to change background color of marked property on ARD graph
		JButton ARDMarkedPropertyBackgroundColorButton = new JButton("Marked property background");
		
		ARDMarkedPropertyBackgroundColorButton.addActionListener(new
				ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						Color newTempARDMarkedBackgroundColor = JColorChooser.showDialog(dialog, "Marked ARD background color chooser", tempARDMarkedBackgroundColor);
				
						if (newTempARDMarkedBackgroundColor != null) 
						{
							tempARDMarkedBackgroundColor = newTempARDMarkedBackgroundColor;
		        	
							repaintGraphs();
						}
					}
				});
		
		ARDPropertyBackgroundColorsPanel.add(ARDMarkedPropertyBackgroundColorButton);
		
		// button used to change background color of unmarked property on ARD graph
		JButton ARDUnmarkedPropertyBackgroundColorButton = new JButton("Unmarked property background");
		
		ARDUnmarkedPropertyBackgroundColorButton.addActionListener(new
				ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						Color newTempARDUnmarkedBackgroundColor = JColorChooser.showDialog(dialog, "Unmarked ARD background color chooser", tempARDUnmarkedBackgroundColor);
						
						if (newTempARDUnmarkedBackgroundColor != null) 
						{
							tempARDUnmarkedBackgroundColor = newTempARDUnmarkedBackgroundColor;
        	
							repaintGraphs();
						}
					}
				});
		
		ARDPropertyBackgroundColorsPanel.add(ARDUnmarkedPropertyBackgroundColorButton);
		
		ARDButtonsPanel.add(ARDPropertyBackgroundColorsPanel, BorderLayout.CENTER);
		
		
		JPanel ARDGradientPanel = new JPanel();
		
		// button used to change background color ARD graph
		JButton ARDBackgroundColorButton = new JButton("ARD background");
		
		ARDBackgroundColorButton.addActionListener(new
				ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						Color newTempARDBackgroundColor = JColorChooser.showDialog(dialog, "ARD background color chooser", tempARDBackgroundColor);
						
						if (newTempARDBackgroundColor != null) 
						{
							tempARDBackgroundColor = newTempARDBackgroundColor;
        	
							repaintGraphs();
						}
					}
				});
		
		ARDGradientPanel.add(ARDBackgroundColorButton);
		
		ARDGradientPanel.add(new JLabel("        "));
		
		
		ARDGradientCheckBox = new JCheckBox("Use gradient");
		ARDGradientCheckBox.setSelected(arranger.isGradientARDCell());
		
		ARDGradientCheckBox.addActionListener(new
				ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						tempIsGradientARDCell = ((JCheckBox)e.getSource()).isSelected();
						repaintGraphs();
					}
				});
		
		
		ARDGradientPanel.add(ARDGradientCheckBox, BorderLayout.SOUTH);
		
		ARDButtonsPanel.add(ARDGradientPanel, BorderLayout.SOUTH);
		
		ARDColorsPanel.add(ARDButtonsPanel, BorderLayout.SOUTH);
		
		ARDColorsPanel.setBorder(BorderFactory.createTitledBorder("ARD "));
		
		mainPanel.add(ARDColorsPanel, BorderLayout.WEST);
		
		
		// TPH panel
		JPanel TPHColorsPanel = new JPanel();
		
		TPHColorsPanel.setLayout(new BorderLayout());
		
		TPHColorsPanel.setPreferredSize(new Dimension(400, 450));
		
		List<DefaultGraphCell> TPHCellsList = new ArrayList<DefaultGraphCell>();
		
		firstTPHMarkedNode = new GraphNode(new Property(firstMarkedPropertyAttributes));
		secondTPHMarkedNode = new GraphNode(new Property(secondMarkedPropertyAttributes));
		unmarkedTPHNode = new GraphNode(new Property(unmarkedPropertyAttributes));
		
		TPHCellsList.add(firstTPHMarkedNode);
		TPHCellsList.add(secondTPHMarkedNode);
		TPHCellsList.add(unmarkedTPHNode);
		
		markedTPHEdge = new DefaultEdge();
		
		markedTPHEdge.setSource(firstTPHMarkedNode.getChildAt(0));
		markedTPHEdge.setTarget(secondTPHMarkedNode.getChildAt(0));
		
		TPHCellsList.add(markedTPHEdge);
		
		unmarkedTPHEdge = new DefaultEdge();
		
		unmarkedTPHEdge.setSource(firstTPHMarkedNode.getChildAt(0));
		unmarkedTPHEdge.setTarget(unmarkedTPHNode.getChildAt(0));
		
		TPHCellsList.add(unmarkedTPHEdge);
		
		DefaultGraphCell[] TPHCells = new DefaultGraphCell[TPHCellsList.size()];
		
		for (int i = 0 ; i < TPHCellsList.size() ; i++)
		{
			TPHCells[i] = TPHCellsList.get(i);
		}
		
		TPHGraph = new JGraph(new DefaultGraphModel());
		
		TPHGraph.getGraphLayoutCache().insert(TPHCells);
		
		Object TPHRoots[] = new Object[1];
		TPHRoots[0] = TPHCells[0];
		
		facade = new JGraphFacade(TPHGraph, TPHRoots);
		
		graphTreeLayout.run(facade);

		nested = facade.createNestedMap(true, true);
		
		TPHGraph.getGraphLayoutCache().edit(nested);
		
		for (int i = 0 ; i < TPHCells.length ; i++)
		{
			GraphConstants.setSelectable(TPHCells[i].getAttributes(), false);
		}

		cds.jlow.client.graph.GraphJ.move(TPHGraph, TPHCells, 50, 100);
		
		TPHColorsPanel.add(TPHGraph, BorderLayout.CENTER);
		
		
		JPanel TPHButtonsPanel = new JPanel();
		
		TPHButtonsPanel.setLayout(new BorderLayout());
		
		
		JPanel TPHLineColorsPanel = new JPanel();
		
		JButton TPHMarkedLineColorButton = new JButton("Marked line color");
		
		TPHMarkedLineColorButton.addActionListener(new
				ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						Color newTempTPHMarkedLineColor = JColorChooser.showDialog(dialog, "Marked TPH line color chooser", tempARDMarkedLineColor);
						
				        if (newTempTPHMarkedLineColor != null) 
				        {
				        	tempTPHMarkedLineColor = newTempTPHMarkedLineColor;
				        	
				        	repaintGraphs();
				        }
					}
				});
		
		TPHLineColorsPanel.add(TPHMarkedLineColorButton);
		
		JButton TPHUnmarkedLineColorButton = new JButton("Unmarked line color");
		
		TPHUnmarkedLineColorButton.addActionListener(new
				ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						Color newTempTPHUnmarkedLineColor = JColorChooser.showDialog(dialog, "Unmarked TPH line color chooser", tempARDUnmarkedLineColor);
						
				        if (newTempTPHUnmarkedLineColor != null) 
				        {
				        	tempTPHUnmarkedLineColor = newTempTPHUnmarkedLineColor;
				        	
				        	repaintGraphs();
				        }
					}
				});
		
		TPHLineColorsPanel.add(TPHUnmarkedLineColorButton);
		
		
		TPHButtonsPanel.add(TPHLineColorsPanel, BorderLayout.NORTH);
		
		
		JPanel TPHPropertyBackgroundColorsPanel = new JPanel();
		
		JButton TPHMarkedPropertyBackgroundColorButton = new JButton("Marked property background");
		
		TPHMarkedPropertyBackgroundColorButton.addActionListener(new
				ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						Color newTempTPHMarkedBackgroundColor = JColorChooser.showDialog(dialog, "Marked TPH background color chooser", tempARDMarkedBackgroundColor);
						
						if (newTempTPHMarkedBackgroundColor != null) 
						{
							tempTPHMarkedBackgroundColor = newTempTPHMarkedBackgroundColor;
		        	
							repaintGraphs();
						}
					}
				});
		
		TPHPropertyBackgroundColorsPanel.add(TPHMarkedPropertyBackgroundColorButton);
		
		JButton TPHUnmarkedPropertyBackgroundColorButton = new JButton("Unmarked property background");
		
		TPHUnmarkedPropertyBackgroundColorButton.addActionListener(new
				ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						Color newTempTPHUnmarkedBackgroundColor = JColorChooser.showDialog(dialog, "Unmarked TPH background color chooser", tempARDUnmarkedBackgroundColor);
						
						if (newTempTPHUnmarkedBackgroundColor != null) 
						{
							tempTPHUnmarkedBackgroundColor = newTempTPHUnmarkedBackgroundColor;
        	
							repaintGraphs();
						}
					}
				});
		
		TPHPropertyBackgroundColorsPanel.add(TPHUnmarkedPropertyBackgroundColorButton);
		
		
		TPHButtonsPanel.add(TPHPropertyBackgroundColorsPanel, BorderLayout.CENTER);
		
		
		JPanel TPHGradientPanel = new JPanel();
		
		JButton TPHBackgroundColorButton = new JButton("TPH background");
		
		TPHBackgroundColorButton.addActionListener(new
				ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						Color newTempTPHBackgroundColor = JColorChooser.showDialog(dialog, "TPH background color chooser", tempARDBackgroundColor);
						
						if (newTempTPHBackgroundColor != null) 
						{
							tempTPHBackgroundColor = newTempTPHBackgroundColor;
        	
							repaintGraphs();
						}
					}
				});
		
		TPHGradientPanel.add(TPHBackgroundColorButton);
		
		TPHGradientPanel.add(new JLabel("        "));
		
		
		TPHGradientCheckBox = new JCheckBox("Use gradient");
		TPHGradientCheckBox.setSelected(arranger.isGradientTPHCell());
		
		TPHGradientCheckBox.addActionListener(new
				ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						tempIsGradientTPHCell = ((JCheckBox)e.getSource()).isSelected();
						repaintGraphs();
					}
				});
		
		TPHGradientPanel.add(TPHGradientCheckBox, BorderLayout.SOUTH);
		
		TPHButtonsPanel.add(TPHGradientPanel, BorderLayout.SOUTH);
		
		TPHColorsPanel.add(TPHButtonsPanel, BorderLayout.SOUTH);
		
		TPHColorsPanel.setBorder(BorderFactory.createTitledBorder("TPH "));
		
		mainPanel.add(TPHColorsPanel, BorderLayout.EAST);
		
		
		this.add(mainPanel);
		
		
		
		JPanel buttonsPanel = new JPanel();
		
		// button used to reset colors
		JButton resetButton = new JButton("Reset");
		
		resetButton.addActionListener(new
				ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						resetColors();
					}
				});
		
		buttonsPanel.add(resetButton);
		
		
		// button used to close window
		JButton cancelButton = new JButton("Cancel");
			
		cancelButton.addActionListener(new
				ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						ok = false;
						dialog.setVisible(false);
					}
				});
		
		buttonsPanel.add(cancelButton);
		
		
		// button used to change colors
		JButton OKButton = new JButton("OK");
		
		OKButton.addActionListener(new
				ActionListener()
				{
					public void actionPerformed(ActionEvent e)
					{
						saveColors();
				
						ok = true;
						dialog.setVisible(false);
					}
				});
		
		buttonsPanel.add(OKButton);
		
		
		this.add(buttonsPanel, BorderLayout.SOUTH);
		
		repaintGraphs();
	}
	
	
	/**
	 * Method used to repaint both ARD and TPH graphs with new colors
	 */
	private void repaintGraphs()
	{
		// ARD graph
		ARDGraph.setBackground(tempARDBackgroundColor);
		
		Map nested=new Hashtable();
		
		Map markedARDLineMap = new Hashtable();

		GraphConstants.setLineEnd(markedARDLineMap, GraphConstants.ARROW_CLASSIC);
		GraphConstants.setEndFill(markedARDLineMap, true);
		GraphConstants.setLineColor(markedARDLineMap, tempARDMarkedLineColor);

		nested.put(markedARDEdge, markedARDLineMap);
		
		
		Map unmarkedARDLineMap = new Hashtable();

		GraphConstants.setLineEnd(unmarkedARDLineMap, GraphConstants.ARROW_CLASSIC);
		GraphConstants.setEndFill(unmarkedARDLineMap, true);
		GraphConstants.setLineColor(unmarkedARDLineMap, tempARDUnmarkedLineColor);
		
		nested.put(unmarkedARDEdge, unmarkedARDLineMap);
		
		
		Map markedARDPropertyMap = new Hashtable();

		if (!tempIsGradientARDCell)
		{
			GraphConstants.setGradientColor(markedARDPropertyMap, tempARDMarkedBackgroundColor);
			GraphConstants.setBackground(markedARDPropertyMap, tempARDMarkedBackgroundColor);
			GraphConstants.setInset(markedARDPropertyMap, 10);
			GraphConstants.setBorderColor(markedARDPropertyMap, tempARDMarkedLineColor);
			GraphConstants.setAutoSize(markedARDPropertyMap, true);
			GraphConstants.setOpaque(markedARDPropertyMap, true);
		}
		else
		{
			GraphConstants.setBackground(markedARDPropertyMap, Color.WHITE);
			GraphConstants.setGradientColor(markedARDPropertyMap, tempARDMarkedBackgroundColor);
			GraphConstants.setOpaque(markedARDPropertyMap, true);
			GraphConstants.setInset(markedARDPropertyMap, 10);
			GraphConstants.setBorderColor(markedARDPropertyMap, tempARDMarkedLineColor);
			GraphConstants.setAutoSize(markedARDPropertyMap, true);
		}

		nested.put(firstARDMarkedNode, markedARDPropertyMap);
		nested.put(secondARDMarkedNode, markedARDPropertyMap);
		
		
		Map unmarkedARDPropertyMap = new Hashtable();
		
		if (!tempIsGradientARDCell)
		{
			GraphConstants.setGradientColor(unmarkedARDPropertyMap, tempARDUnmarkedBackgroundColor);
			GraphConstants.setBackground(unmarkedARDPropertyMap, tempARDUnmarkedBackgroundColor);
			GraphConstants.setInset(unmarkedARDPropertyMap, 10);
			GraphConstants.setBorderColor(unmarkedARDPropertyMap, tempARDUnmarkedLineColor);
			GraphConstants.setAutoSize(unmarkedARDPropertyMap, true);
			GraphConstants.setOpaque(unmarkedARDPropertyMap, true);
		}
		else
		{
			GraphConstants.setBackground(unmarkedARDPropertyMap, Color.WHITE);
			GraphConstants.setGradientColor(unmarkedARDPropertyMap, tempARDUnmarkedBackgroundColor);
			GraphConstants.setOpaque(unmarkedARDPropertyMap, true);
			GraphConstants.setInset(unmarkedARDPropertyMap, 10);
			GraphConstants.setBorderColor(unmarkedARDPropertyMap, tempARDUnmarkedLineColor);
			GraphConstants.setAutoSize(unmarkedARDPropertyMap, true);
		}
		
		nested.put(unmarkedARDNode, unmarkedARDPropertyMap);
		
		ARDGraph.getGraphLayoutCache().edit(nested,null,null,null);
		
		
		// TPH graph
		TPHGraph.setBackground(tempTPHBackgroundColor);
		
		nested=new Hashtable();
		
		Map markedTPHLineMap = new Hashtable();

		GraphConstants.setLineEnd(markedTPHLineMap, GraphConstants.ARROW_CLASSIC);
		GraphConstants.setEndFill(markedTPHLineMap, true);
		GraphConstants.setLineColor(markedTPHLineMap, tempTPHMarkedLineColor);

		nested.put(markedTPHEdge, markedTPHLineMap);
		
		
		Map unmarkedTPHLineMap = new Hashtable();

		GraphConstants.setLineEnd(unmarkedTPHLineMap, GraphConstants.ARROW_CLASSIC);
		GraphConstants.setEndFill(unmarkedTPHLineMap, true);
		GraphConstants.setLineColor(unmarkedTPHLineMap, tempTPHUnmarkedLineColor);
		
		nested.put(unmarkedTPHEdge, unmarkedTPHLineMap);
		
		
		Map markedTPHPropertyMap = new Hashtable();
		
		if (!tempIsGradientTPHCell)
		{
			GraphConstants.setGradientColor(markedTPHPropertyMap, tempTPHMarkedBackgroundColor);
			GraphConstants.setBackground(markedTPHPropertyMap, tempTPHMarkedBackgroundColor);
			GraphConstants.setInset(markedTPHPropertyMap, 10);
			GraphConstants.setBorderColor(markedTPHPropertyMap, tempTPHMarkedLineColor);
			GraphConstants.setAutoSize(markedTPHPropertyMap, true);
			GraphConstants.setOpaque(markedTPHPropertyMap, true);
		}
		else
		{
			GraphConstants.setBackground(markedTPHPropertyMap, Color.WHITE);
			GraphConstants.setGradientColor(markedTPHPropertyMap, tempTPHMarkedBackgroundColor);
			GraphConstants.setOpaque(markedTPHPropertyMap, true);
			GraphConstants.setInset(markedTPHPropertyMap, 10);
			GraphConstants.setBorderColor(markedTPHPropertyMap, tempTPHMarkedLineColor);
			GraphConstants.setAutoSize(markedTPHPropertyMap, true);
		}

		nested.put(firstTPHMarkedNode, markedTPHPropertyMap);
		nested.put(secondTPHMarkedNode, markedTPHPropertyMap);
		
		
		Map unmarkedTPHPropertyMap = new Hashtable();
		
		if (!tempIsGradientTPHCell)
		{
			GraphConstants.setGradientColor(unmarkedTPHPropertyMap, tempTPHUnmarkedBackgroundColor);
			GraphConstants.setBackground(unmarkedTPHPropertyMap, tempTPHUnmarkedBackgroundColor);
			GraphConstants.setInset(unmarkedTPHPropertyMap, 10);
			GraphConstants.setBorderColor(unmarkedTPHPropertyMap, tempTPHUnmarkedLineColor);
			GraphConstants.setAutoSize(unmarkedTPHPropertyMap, true);
			GraphConstants.setOpaque(unmarkedTPHPropertyMap, true);
		}
		else
		{
			GraphConstants.setBackground(unmarkedTPHPropertyMap, Color.WHITE);
			GraphConstants.setGradientColor(unmarkedTPHPropertyMap, tempTPHUnmarkedBackgroundColor);
			GraphConstants.setOpaque(unmarkedTPHPropertyMap, true);
			GraphConstants.setInset(unmarkedTPHPropertyMap, 10);
			GraphConstants.setBorderColor(unmarkedTPHPropertyMap, tempTPHUnmarkedLineColor);
			GraphConstants.setAutoSize(unmarkedTPHPropertyMap, true);
		}

		nested.put(unmarkedTPHNode, unmarkedTPHPropertyMap);
		
		TPHGraph.getGraphLayoutCache().edit(nested,null,null,null);
		
		ARDGraph.repaint();
		TPHGraph.repaint();
	}
	
	/**
	 * Method used to reset colors
	 */
	private void resetColors()
	{
		tempARDMarkedLineColor = arranger.getARDMarkedLineColor();
		tempARDUnmarkedLineColor = arranger.getARDUnmarkedLineColor();
		tempARDMarkedBackgroundColor = arranger.getARDMarkedBackgroundColor();
		tempARDUnmarkedBackgroundColor = arranger.getARDUnmarkedBackgroundColor();
		tempARDBackgroundColor = arranger.getARDBackgroundColor();
		
		tempTPHMarkedLineColor = arranger.getTPHMarkedLineColor();
		tempTPHUnmarkedLineColor = arranger.getTPHUnmarkedLineColor();
		tempTPHMarkedBackgroundColor = arranger.getTPHMarkedBackgroundColor();
		tempTPHUnmarkedBackgroundColor = arranger.getTPHUnmarkedBackgroundColor();
		tempTPHBackgroundColor = arranger.getTPHBackgroundColor();
		
		tempIsGradientARDCell = arranger.isGradientARDCell();
		tempIsGradientTPHCell = arranger.isGradientTPHCell();
		
		ARDGradientCheckBox.setSelected(tempIsGradientARDCell);
		TPHGradientCheckBox.setSelected(tempIsGradientTPHCell);
		
		validate();
		repaintGraphs();
	}
	
	
	/**
	 * Method used to save new colors
	 */
	private void saveColors()
	{
		arranger.setARDMarkedLineColor(tempARDMarkedLineColor);
		arranger.setARDUnmarkedLineColor(tempARDUnmarkedLineColor);
		arranger.setARDMarkedBackgroundColor(tempARDMarkedBackgroundColor);
		arranger.setARDUnmarkedBackgroundColor(tempARDUnmarkedBackgroundColor);
		arranger.setARDBackgroundColor(tempARDBackgroundColor);
		
		arranger.setTPHMarkedLineColor(tempTPHMarkedLineColor);
		arranger.setTPHUnmarkedLineColor(tempTPHUnmarkedLineColor);
		arranger.setTPHMarkedBackgroundColor(tempTPHMarkedBackgroundColor);
		arranger.setTPHUnmarkedBackgroundColor(tempTPHUnmarkedBackgroundColor);
		arranger.setTPHBackgroundColor(tempTPHBackgroundColor);
		
		arranger.setGradientARDCell(tempIsGradientARDCell);
		arranger.setGradientTPHCell(tempIsGradientTPHCell);
	}

	
	public boolean showDialog(String title)
	{
		ok = false;
		
		dialog = new JDialog(new JFrame(), title, true);
		
		int screenWidth = (int)Toolkit.getDefaultToolkit().getScreenSize().getWidth();
		int screenHeight = (int)Toolkit.getDefaultToolkit().getScreenSize().getHeight();
		
		dialog.getContentPane().add(this);
		
		dialog.pack();
		
		dialog.setLocation(screenWidth/2 - dialog.getWidth()/2, screenHeight/2 - dialog.getHeight()/2);
		
		dialog.setResizable(false);
		
		dialog.setVisible(true);
		return ok;
	}
}